@@ -116,8 +116,8 @@ class LudwigModel:
         of backend to use to execute preprocessing / training steps.
     :param gpus: (Union[str, int, List[int]], default: `None`) GPUs
         to use (it uses the same syntax of CUDA_VISIBLE_DEVICES)
-    :param gpu_memory_limit: (int: default: `None`) maximum memory in MB to
-        allocate per GPU device.
+    :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+        [0, 1] allowed to allocate per GPU device.
     :param allow_parallel_threads: (bool, default: `True`) allow Torch
         to use multithreading parallelism to improve performance at the
         cost of determinism.
@@ -179,7 +179,7 @@ class LudwigModel:
         logging_level: int = logging.ERROR,
         backend: Union[Backend, str] = None,
         gpus: Union[str, int, List[int]] = None,
-        gpu_memory_limit: int = None,
+        gpu_memory_limit: Optional[float] = None,
         allow_parallel_threads: bool = True,
         callbacks: List[Callback] = None,
     ) -> None:
@@ -194,8 +194,8 @@ class LudwigModel:
             of backend to use to execute preprocessing / training steps.
         :param gpus: (Union[str, int, List[int]], default: `None`) GPUs
             to use (it uses the same syntax of CUDA_VISIBLE_DEVICES)
-        :param gpu_memory_limit: (int: default: `None`) maximum memory in MB to
-            allocate per GPU device.
+        :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+            [0, 1] allowed to allocate per GPU device.
         :param allow_parallel_threads: (bool, default: `True`) allow Torch
             to use multithreading parallelism to improve performance at the
             cost of determinism.
@@ -602,7 +602,7 @@ class LudwigModel:
 
                     # save training statistics
                     if self.backend.is_coordinator():
-                        if not skip_save_training_statistics and path_exists(os.path.dirname(training_stats_fn)):
+                        if not skip_save_training_statistics:
                             save_json(training_stats_fn, train_stats)
 
                     # results of the model with highest validation test performance
@@ -1316,7 +1316,7 @@ class LudwigModel:
         logging_level: int = logging.ERROR,
         backend: Union[Backend, str] = None,
         gpus: Union[str, int, List[int]] = None,
-        gpu_memory_limit: int = None,
+        gpu_memory_limit: Optional[float] = None,
         allow_parallel_threads: bool = True,
         callbacks: List[Callback] = None,
     ) -> "LudwigModel":  # return is an instance of ludwig.api.LudwigModel class
@@ -1333,8 +1333,8 @@ class LudwigModel:
             of backend to use to execute preprocessing / training steps.
         :param gpus: (Union[str, int, List[int]], default: `None`) GPUs
             to use (it uses the same syntax of CUDA_VISIBLE_DEVICES)
-        :param gpu_memory_limit: (int: default: `None`) maximum memory in MB to
-            allocate per GPU device.
+        :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+            [0, 1] allowed to allocate per GPU device.
         :param allow_parallel_threads: (bool, default: `True`) allow Torch
             to use
             multithreading parallelism to improve performance at the cost of
@@ -1579,7 +1579,7 @@ def kfold_cross_validate(
     output_directory: str = "results",
     random_seed: int = default_random_seed,
     gpus: Union[str, int, List[int]] = None,
-    gpu_memory_limit: int = None,
+    gpu_memory_limit: Optional[float] = None,
     allow_parallel_threads: bool = True,
     backend: Union[Backend, str] = None,
     logging_level: int = logging.INFO,
@@ -1649,8 +1649,8 @@ def kfold_cross_validate(
            splits and any other random function.
     :param gpus: (list, default: `None`) list of GPUs that are available
             for training.
-    :param gpu_memory_limit: (int, default: `None`) maximum memory in MB to
-            allocate per GPU device.
+    :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+            [0, 1] allowed to allocate per GPU device.
     :param allow_parallel_threads: (bool, default: `True`) allow Torch to
             use multithreading parallelism
            to improve performance at the cost of determinism.
