@@ -12,12 +12,11 @@ import copy
 import logging
 import os
 import warnings
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import numpy as np
 import pandas as pd
 import yaml
-from packaging.version import parse as parse_version
 
 from ludwig.api import LudwigModel
 from ludwig.automl.auto_tune_config import memory_tune_config
@@ -42,7 +41,7 @@ from ludwig.utils.automl.utils import (
     has_imbalanced_output,
     set_output_feature_metric,
 )
-from ludwig.utils.data_utils import load_dataset
+from ludwig.utils.data_utils import load_dataset, use_credentials
 from ludwig.utils.defaults import default_random_seed
 from ludwig.utils.fs_utils import open_file
 from ludwig.utils.misc_utils import merge_dict
@@ -52,10 +51,8 @@ try:
     import dask.dataframe as dd
     import ray
     from ray.tune import ExperimentAnalysis
-
-    _ray_113 = parse_version(ray.__version__) >= parse_version("1.13.0")
-except ImportError:
-    raise ImportError(" ray is not installed. In order to use auto_train please run pip install ludwig[ray]")
+except ImportError as e:
+    raise RuntimeError("ray is not installed. In order to use auto_train please run pip install ludwig[ray]") from e
 
 
 logger = logging.getLogger(__name__)
@@ -64,8 +61,9 @@ OUTPUT_DIR = "."
 
 
 class AutoTrainResults:
-    def __init__(self, experiment_analysis: ExperimentAnalysis):
+    def __init__(self, experiment_analysis: ExperimentAnalysis, creds: Dict[str, Any] = None):
         self._experiment_analysis = experiment_analysis
+        self._creds = creds
 
     @property
     def experiment_analysis(self):
@@ -82,11 +80,15 @@ class AutoTrainResults:
             logger.warning("No best model found")
             return None
 
-        if not _ray_113:
-            return LudwigModel.load(os.path.join(checkpoint, "model"))
-
-        with checkpoint.as_directory() as checkpoint:
-            return LudwigModel.load(os.path.join(checkpoint, "model"))
+        ckpt_type, ckpt_path = checkpoint.get_internal_representation()
+        if ckpt_type == "uri":
+            # Read remote URIs using Ludwig's internal remote file loading APIs, as
+            # Ray's do not handle custom credentials at the moment.
+            with use_credentials(self._creds):
+                return LudwigModel.load(os.path.join(ckpt_path, "model"))
+        else:
+            with checkpoint.as_directory() as ckpt_path:
+                return LudwigModel.load(os.path.join(ckpt_path, "model"))
 
 
 def auto_train(
@@ -236,8 +238,14 @@ def train_with_config(
                 "Consider increasing the time budget for experiment. "
             )
 
+    # Extract credentials needed to pull artifacts, if provided
+    creds = None
+    backend: Backend = initialize_backend(kwargs.get("backend"))
+    if backend is not None:
+        creds = backend.storage.artifacts.credentials
+
     experiment_analysis = hyperopt_results.experiment_analysis
-    return AutoTrainResults(experiment_analysis)
+    return AutoTrainResults(experiment_analysis, creds)
 
 
 def _model_select(
