@@ -46,6 +46,10 @@ class DataFrameEngine(ABC):
     def map_partitions(self, series, map_fn, meta=None):
         raise NotImplementedError()
 
+    @abstractmethod
+    def map_batches(self, df, map_fn):
+        raise NotImplementedError()
+
     @abstractmethod
     def apply_objects(self, series, map_fn, meta=None):
         raise NotImplementedError()
@@ -55,7 +59,16 @@ class DataFrameEngine(ABC):
         raise NotImplementedError()
 
     @abstractmethod
-    def to_parquet(self, df, path):
+    def split(self, df, probabilities):
+        """Splits the input DataFrame into sections with the given proportions."""
+        raise NotImplementedError()
+
+    @abstractmethod
+    def to_parquet(self, df, path, index=False):
+        """Write the input DataFrame to the path in the Parquet format.
+
+        Optionally includes the DataFrame index in the Parquet file.
+        """
         raise NotImplementedError()
 
     @abstractmethod
