@@ -18,6 +18,7 @@ import modin.pandas as pd
 import numpy as np
 
 from ludwig.data.dataframe.base import DataFrameEngine
+from ludwig.utils.data_utils import split_by_slices
 
 
 class ModinEngine(DataFrameEngine):
@@ -38,11 +39,14 @@ class ModinEngine(DataFrameEngine):
         return data
 
     def from_pandas(self, df):
-        return df
+        return pd.DataFrame(df)
 
     def map_objects(self, series, map_fn, meta=None):
         return series.map(map_fn)
 
+    def map_batches(self, df, map_fn):
+        return map_fn(df)
+
     def map_partitions(self, series, map_fn, meta=None):
         return map_fn(series)
 
@@ -52,8 +56,14 @@ class ModinEngine(DataFrameEngine):
     def reduce_objects(self, series, reduce_fn):
         return reduce_fn(series)
 
-    def to_parquet(self, df, path):
-        df.to_parquet(path, engine="pyarrow")
+    def split(self, df, probabilities):
+        return split_by_slices(df.iloc, len(df), probabilities)
+
+    def remove_empty_partitions(self, df):
+        return df
+
+    def to_parquet(self, df, path, index=False):
+        df.to_parquet(path, engine="pyarrow", index=index)
 
     def to_ray_dataset(self, df):
         from ray.data import from_modin
@@ -63,6 +73,9 @@ class ModinEngine(DataFrameEngine):
     def from_ray_dataset(self, dataset) -> pd.DataFrame:
         return dataset.to_modin()
 
+    def reset_index(self, df):
+        return df.reset_index(drop=True)
+
     @property
     def array_lib(self):
         return np
