@@ -22,6 +22,7 @@ from ludwig.constants import CATEGORY
 from ludwig.encoders.base import Encoder
 from ludwig.encoders.registry import register_encoder
 from ludwig.modules.embedding_modules import Embed
+from ludwig.schema.encoders.category_encoders import CategoricalEmbedConfig, CategoricalSparseConfig
 
 logger = logging.getLogger(__name__)
 
@@ -37,9 +38,12 @@ class CategoricalEmbedEncoder(Encoder):
         embeddings_on_cpu: bool = False,
         dropout: float = 0.0,
         embedding_initializer: Optional[Union[str, Dict]] = None,
+        encoder_config=None,
         **kwargs,
     ):
         super().__init__()
+        self.config = encoder_config
+
         logger.debug(f" {self.name}")
 
         logger.debug("  Embed")
@@ -65,6 +69,10 @@ class CategoricalEmbedEncoder(Encoder):
         embedded = self.embed(inputs)
         return embedded
 
+    @staticmethod
+    def get_schema_cls():
+        return CategoricalEmbedConfig
+
     @property
     def output_shape(self) -> torch.Size:
         return torch.Size([self.embedding_size])
@@ -84,9 +92,12 @@ class CategoricalSparseEncoder(Encoder):
         embeddings_on_cpu: bool = False,
         dropout: float = 0.0,
         embedding_initializer: Optional[Union[str, Dict]] = None,
+        encoder_config=None,
         **kwargs,
     ):
         super().__init__()
+        self.config = encoder_config
+
         logger.debug(f" {self.name}")
 
         logger.debug("  Embed")
@@ -112,6 +123,10 @@ class CategoricalSparseEncoder(Encoder):
         embedded = self.embed(inputs)
         return embedded
 
+    @staticmethod
+    def get_schema_cls():
+        return CategoricalSparseConfig
+
     @property
     def output_shape(self) -> torch.Size:
         return torch.Size([self.embedding_size])
