@@ -16,7 +16,7 @@
 import argparse
 import logging
 import sys
-from typing import List, Union
+from typing import List, Optional, Union
 
 import pandas as pd
 
@@ -44,7 +44,7 @@ def evaluate_cli(
     skip_collect_overall_stats: bool = False,
     output_directory: str = "results",
     gpus: Union[str, int, List[int]] = None,
-    gpu_memory_limit: int = None,
+    gpu_memory_limit: Optional[float] = None,
     allow_parallel_threads: bool = True,
     callbacks: List[Callback] = None,
     backend: Union[Backend, str] = None,
@@ -87,8 +87,8 @@ def evaluate_cli(
          model and the training progress files.
      :param gpus: (list, default: `None`) list of GPUs that are available
          for training.
-     :param gpu_memory_limit: (int, default: `None`) maximum memory in MB to
-         allocate per GPU device.
+     :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+            [0, 1] allowed to allocate per GPU device.
      :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
          to use multithreading parallelism to improve performance at
          the cost of determinism.
@@ -212,7 +212,11 @@ def cli(sys_argv):
     # ------------------
     parser.add_argument("-g", "--gpus", type=int, default=0, help="list of gpu to use")
     parser.add_argument(
-        "-gml", "--gpu_memory_limit", type=int, default=None, help="maximum memory in MB to allocate per GPU device"
+        "-gml",
+        "--gpu_memory_limit",
+        type=float,
+        default=None,
+        help="maximum memory fraction [0, 1] allowed to allocate per GPU device",
     )
     parser.add_argument(
         "-dpt",
