@@ -23,9 +23,9 @@ import torchaudio
 
 from ludwig.constants import (
     AUDIO,
-    BACKFILL,
+    AUDIO_FEATURE_KEYS,
     COLUMN,
-    MISSING_VALUE_STRATEGY_OPTIONS,
+    ENCODER,
     NAME,
     PREPROCESSING,
     PROC_COLUMN,
@@ -35,6 +35,8 @@ from ludwig.constants import (
 )
 from ludwig.features.base_feature import BaseFeatureMixin
 from ludwig.features.sequence_feature import SequenceInputFeature
+from ludwig.schema.features.audio_feature import AudioInputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature
 from ludwig.utils.audio_utils import (
     calculate_mean,
     calculate_var,
@@ -46,13 +48,16 @@ from ludwig.utils.audio_utils import (
     get_non_symmetric_length,
     get_phase_stft_magnitude,
     get_stft_magnitude,
-    read_audio_if_bytes_obj,
+    is_torch_audio_tuple,
+    read_audio_from_bytes_obj,
 )
 from ludwig.utils.data_utils import get_abs_path
 from ludwig.utils.fs_utils import has_remote_protocol
 from ludwig.utils.misc_utils import set_default_value, set_default_values
 from ludwig.utils.types import TorchscriptPreprocessingInput
 
+logger = logging.getLogger(__name__)
+
 
 class _AudioPreprocessing(torch.nn.Module):
 
@@ -60,7 +65,11 @@ class _AudioPreprocessing(torch.nn.Module):
 
     def __init__(self, metadata: Dict[str, Any]):
         super().__init__()
-        self.audio_feature_dict = metadata["preprocessing"]["audio_feature"]
+        self.audio_feature_dict = {
+            key: value
+            for key, value in metadata["preprocessing"].items()
+            if key in AUDIO_FEATURE_KEYS and value is not None
+        }
         self.feature_dim = metadata["feature_dim"]
         self.max_length = metadata["max_length"]
         self.padding_value = metadata["preprocessing"]["padding_value"]
@@ -92,40 +101,7 @@ class AudioFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "audio_file_length_limit_in_s": 7.5,
-            "missing_value_strategy": BACKFILL,
-            "in_memory": True,
-            "padding_value": 0,
-            "norm": None,
-            "audio_feature": {
-                TYPE: "fbank",
-                "window_length_in_s": 0.04,
-                "window_shift_in_s": 0.02,
-                "num_filter_bands": 80,
-            },
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "audio_file_length_limit_in_s": {"type": "number", "minimum": 0},
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "in_memory": {"type": "boolean"},
-            "padding_value": {"type": "number", "minimum": 0},
-            "norm": {"type": ["string", "null"], "enum": [None, "per_file", "global"]},
-            "audio_feature": {
-                "type": "object",
-                "properties": {
-                    "type": {"type": "string", "enum": ["raw", "stft", "stft_phase", "group_delay", "fbank"]},
-                    "window_length_in_s": {"type": "number", "minimum": 0},
-                    "window_shift_in_s": {"type": "number", "minimum": 0},
-                    "num_fft_points": {"type": "number", "minimum": 0},
-                    "window_type": {"type": "string"},
-                    "num_filter_bands": {"type": "number", "minimum": 0},
-                },
-            },
-        }
+        return AudioInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -133,14 +109,13 @@ class AudioFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def get_feature_meta(column, preprocessing_parameters, backend):
-        audio_feature_dict = preprocessing_parameters["audio_feature"]
         first_audio_file_path = column.head(1)[0]
         _, sampling_rate_in_hz = torchaudio.load(first_audio_file_path)
 
-        feature_dim = AudioFeatureMixin._get_feature_dim(audio_feature_dict, sampling_rate_in_hz)
+        feature_dim = AudioFeatureMixin._get_feature_dim(preprocessing_parameters, sampling_rate_in_hz)
         audio_file_length_limit_in_s = preprocessing_parameters["audio_file_length_limit_in_s"]
         max_length = AudioFeatureMixin._get_max_length_feature(
-            audio_feature_dict, sampling_rate_in_hz, audio_file_length_limit_in_s
+            preprocessing_parameters, sampling_rate_in_hz, audio_file_length_limit_in_s
         )
         return {
             "feature_dim": feature_dim,
@@ -150,19 +125,23 @@ class AudioFeatureMixin(BaseFeatureMixin):
         }
 
     @staticmethod
-    def _get_feature_dim(audio_feature_dict, sampling_rate_in_hz):
-        feature_type = audio_feature_dict[TYPE]
+    def _get_feature_dim(preprocessing_parameters, sampling_rate_in_hz):
+        feature_type = preprocessing_parameters[TYPE]
 
         if feature_type == "raw":
             feature_dim = 1
         elif feature_type == "stft_phase":
-            feature_dim_symmetric = get_length_in_samp(audio_feature_dict["window_length_in_s"], sampling_rate_in_hz)
+            feature_dim_symmetric = get_length_in_samp(
+                preprocessing_parameters["window_length_in_s"], sampling_rate_in_hz
+            )
             feature_dim = 2 * get_non_symmetric_length(feature_dim_symmetric)
         elif feature_type in ["stft", "group_delay"]:
-            feature_dim_symmetric = get_length_in_samp(audio_feature_dict["window_length_in_s"], sampling_rate_in_hz)
+            feature_dim_symmetric = get_length_in_samp(
+                preprocessing_parameters["window_length_in_s"], sampling_rate_in_hz
+            )
             feature_dim = get_non_symmetric_length(feature_dim_symmetric)
         elif feature_type == "fbank":
-            feature_dim = audio_feature_dict["num_filter_bands"]
+            feature_dim = preprocessing_parameters["num_filter_bands"]
         else:
             raise ValueError(f"{feature_type} is not recognized.")
 
@@ -181,18 +160,12 @@ class AudioFeatureMixin(BaseFeatureMixin):
     ):
 
         df_engine = backend.df_engine
-        raw_audio = backend.read_binary_files(column, map_fn=read_audio_if_bytes_obj)
-
-        def is_torch_audio_tuple(audio):
-            if isinstance(audio, tuple):
-                if len(audio) == 2 and isinstance(audio[0], torch.Tensor) and isinstance(audio[1], int):
-                    return True
-            return False
+        raw_audio = backend.read_binary_files(column, map_fn=read_audio_from_bytes_obj)
 
         try:
             default_audio = get_default_audio([audio for audio in raw_audio if is_torch_audio_tuple(audio)])
         except RuntimeError:
-            logging.info("Unable to process audio files provided")
+            logger.info("Unable to process audio files provided")
             raise RuntimeError
 
         raw_audio = df_engine.map_objects(raw_audio, lambda row: row if is_torch_audio_tuple(row) else default_audio)
@@ -249,7 +222,7 @@ class AudioFeatureMixin(BaseFeatureMixin):
             merged_stats["cropped"],
             audio_file_length_limit_in_s,
         )
-        logging.debug(print_statistics)
+        logger.debug(print_statistics)
         return processed_audio
 
     @staticmethod
@@ -259,7 +232,7 @@ class AudioFeatureMixin(BaseFeatureMixin):
         audio_feature_dict: Dict[str, Union[float, int, str]],
         feature_dim: int,
         max_length: int,
-        padding_value: int,
+        padding_value: float,
         normalization_type: Optional[str] = None,
         type_key: str = TYPE,
     ):
@@ -285,7 +258,9 @@ class AudioFeatureMixin(BaseFeatureMixin):
 
         feature_length = audio_feature.shape[0]
         broadcast_feature_length = min(feature_length, max_length)
-        audio_feature_padded = torch.full((max_length, feature_dim), padding_value, dtype=torch.float32)
+        audio_feature_padded = torch.full(
+            (max_length, feature_dim), padding_value, dtype=torch.float32, device=audio_feature.device
+        )
         audio_feature_padded[:broadcast_feature_length, :] = audio_feature[:max_length, :]
 
         return audio_feature_padded
@@ -378,11 +353,6 @@ class AudioFeatureMixin(BaseFeatureMixin):
     ):
         set_default_value(feature_config["preprocessing"], "in_memory", preprocessing_parameters["in_memory"])
 
-        if "audio_feature" not in preprocessing_parameters:
-            raise ValueError("audio_feature dictionary has to be present in preprocessing for audio.")
-        if TYPE not in preprocessing_parameters["audio_feature"]:
-            raise ValueError("type has to be present in audio_feature dictionary for audio.")
-
         name = feature_config[NAME]
         column = input_df[feature_config[COLUMN]]
 
@@ -391,7 +361,7 @@ class AudioFeatureMixin(BaseFeatureMixin):
             raise ValueError("There are no audio files in the dataset provided.")
 
         first_audio_entry = next(iter(column))
-        logging.debug(f"Detected audio feature type is {type(first_audio_entry)}")
+        logger.debug(f"Detected audio feature type is {type(first_audio_entry)}")
 
         if not isinstance(first_audio_entry, str) and not isinstance(first_audio_entry, torch.Tensor):
             raise ValueError(
@@ -413,7 +383,11 @@ class AudioFeatureMixin(BaseFeatureMixin):
 
         feature_dim = metadata[name]["feature_dim"]
         max_length = metadata[name]["max_length"]
-        audio_feature_dict = preprocessing_parameters["audio_feature"]
+        audio_feature_dict = {
+            key: value
+            for key, value in preprocessing_parameters.items()
+            if key in AUDIO_FEATURE_KEYS and value is not None
+        }
         audio_file_length_limit_in_s = preprocessing_parameters["audio_file_length_limit_in_s"]
 
         if num_audio_utterances == 0:
@@ -437,8 +411,8 @@ class AudioFeatureMixin(BaseFeatureMixin):
         return proc_df
 
     @staticmethod
-    def _get_max_length_feature(audio_feature_dict, sampling_rate_in_hz, audio_length_limit_in_s):
-        feature_type = audio_feature_dict[TYPE]
+    def _get_max_length_feature(preprocessing_parameters, sampling_rate_in_hz, audio_length_limit_in_s):
+        feature_type = preprocessing_parameters[TYPE]
         audio_length_limit_in_samp = audio_length_limit_in_s * sampling_rate_in_hz
 
         if not audio_length_limit_in_samp.is_integer():
@@ -452,8 +426,8 @@ class AudioFeatureMixin(BaseFeatureMixin):
         if feature_type == "raw":
             return audio_length_limit_in_samp
         elif feature_type in ["stft", "stft_phase", "group_delay", "fbank"]:
-            window_length_in_s = audio_feature_dict["window_length_in_s"]
-            window_shift_in_s = audio_feature_dict["window_shift_in_s"]
+            window_length_in_s = preprocessing_parameters["window_length_in_s"]
+            window_shift_in_s = preprocessing_parameters["window_shift_in_s"]
             return get_max_length_stft_based(
                 audio_length_limit_in_samp, window_length_in_s, window_shift_in_s, sampling_rate_in_hz
             )
@@ -461,16 +435,15 @@ class AudioFeatureMixin(BaseFeatureMixin):
             raise ValueError(f"{feature_type} is not recognized.")
 
 
+@register_input_feature(AUDIO)
 class AudioInputFeature(AudioFeatureMixin, SequenceInputFeature):
-    encoder = "parallel_cnn"
-    max_sequence_length = None
-    embedding_size = None
+    def __init__(self, input_feature_config: Union[AudioInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, encoder_obj=encoder_obj, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature, encoder_obj=encoder_obj)
-        if not self.embedding_size:
+        if not getattr(self.encoder_obj.config, "embedding_size", None):
             raise ValueError("embedding_size has to be defined - " 'check "update_config_with_metadata()"')
-        if not self.max_sequence_length:
+        if not getattr(self.encoder_obj.config, "max_sequence_length", None):
             raise ValueError("max_sequence_length has to be defined - " 'check "update_config_with_metadata()"')
 
     def forward(self, inputs, mask=None):
@@ -484,7 +457,7 @@ class AudioInputFeature(AudioFeatureMixin, SequenceInputFeature):
 
     @property
     def input_shape(self) -> torch.Size:
-        return torch.Size([self.max_sequence_length, self.embedding_size])
+        return torch.Size([self.encoder_obj.config.max_sequence_length, self.encoder_obj.config.embedding_size])
 
     @property
     def input_dtype(self):
@@ -492,14 +465,21 @@ class AudioInputFeature(AudioFeatureMixin, SequenceInputFeature):
 
     @staticmethod
     def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature["max_sequence_length"] = feature_metadata["max_length"]
-        input_feature["embedding_size"] = feature_metadata["feature_dim"]
-        input_feature["should_embed"] = False
+        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_length"]
+        input_feature[ENCODER]["embedding_size"] = feature_metadata["feature_dim"]
+        input_feature[ENCODER]["should_embed"] = False
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_values(input_feature, {TIED: None, "preprocessing": {}})
+        defaults = AudioInputFeatureConfig()
+        set_default_values(
+            input_feature, {ENCODER: {TYPE: defaults.encoder.type}, PREPROCESSING: {}, TIED: defaults.tied}
+        )
 
     @staticmethod
     def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
         return _AudioPreprocessing(metadata)
+
+    @staticmethod
+    def get_schema_cls():
+        return AudioInputFeatureConfig
