@@ -14,7 +14,7 @@
 # limitations under the License.
 # ==============================================================================
 import logging
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Union
 
 import numpy as np
 import torch
@@ -23,24 +23,28 @@ from ludwig.constants import (
     ACCURACY,
     CATEGORY,
     COLUMN,
-    FILL_WITH_CONST,
+    DECODER,
+    DEPENDENCIES,
+    ENCODER,
     HIDDEN,
     HITS_AT_K,
     LOGITS,
     LOSS,
-    MISSING_VALUE_STRATEGY_OPTIONS,
     NAME,
     PREDICTIONS,
     PROBABILITIES,
     PROBABILITY,
     PROC_COLUMN,
     PROJECTION_INPUT,
-    SOFTMAX_CROSS_ENTROPY,
-    SUM,
+    REDUCE_DEPENDENCIES,
+    REDUCE_INPUT,
     TIED,
+    TOP_K,
     TYPE,
 )
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature, OutputFeature, PredictModule
+from ludwig.schema.features.category_feature import CategoryInputFeatureConfig, CategoryOutputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature, register_output_feature
 from ludwig.utils import calibration, output_feature_utils
 from ludwig.utils.eval_utils import ConfusionMatrix
 from ludwig.utils.math_utils import int_type, softmax
@@ -55,7 +59,12 @@ class _CategoryPreprocessing(torch.nn.Module):
     def __init__(self, metadata: Dict[str, Any]):
         super().__init__()
         self.str2idx = metadata["str2idx"]
-        self.unk = self.str2idx[UNKNOWN_SYMBOL]
+        if UNKNOWN_SYMBOL in self.str2idx:
+            self.unk = self.str2idx[UNKNOWN_SYMBOL]
+        else:
+            # self.unk is set to 0 to comply with Torchscript type tracing and will
+            # likely not be used during training, but potentially during inference
+            self.unk = 0
 
     def forward(self, v: TorchscriptPreprocessingInput) -> torch.Tensor:
         if not torch.jit.isinstance(v, List[str]):
@@ -69,16 +78,19 @@ class _CategoryPostprocessing(torch.nn.Module):
     def __init__(self, metadata: Dict[str, Any]):
         super().__init__()
         self.idx2str = {i: v for i, v in enumerate(metadata["idx2str"])}
+        self.unk = UNKNOWN_SYMBOL
         self.predictions_key = PREDICTIONS
         self.probabilities_key = PROBABILITIES
-        self.unk = ""
 
-    def forward(self, preds: Dict[str, torch.Tensor]) -> Dict[str, Any]:
-        predictions = preds[self.predictions_key]
+    def forward(self, preds: Dict[str, torch.Tensor], feature_name: str) -> Dict[str, Any]:
+        predictions = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.predictions_key)
+        probabilities = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.probabilities_key)
+
         inv_preds = [self.idx2str.get(pred, self.unk) for pred in predictions]
+
         return {
             self.predictions_key: inv_preds,
-            self.probabilities_key: preds[self.probabilities_key],
+            self.probabilities_key: probabilities,
         }
 
 
@@ -112,22 +124,7 @@ class CategoryFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "most_common": 10000,
-            "lowercase": False,
-            "missing_value_strategy": FILL_WITH_CONST,
-            "fill_value": UNKNOWN_SYMBOL,
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "most_common": {"type": "integer", "minimum": 0},
-            "lowercase": {"type": "boolean"},
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "fill_value": {"type": "string"},
-            "computed_fill_value": {"type": "string"},
-        }
+        return CategoryInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -145,13 +142,41 @@ class CategoryFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def feature_data(column, metadata):
-        return column.map(
-            lambda x: (
-                metadata["str2idx"][x.strip()]
-                if x.strip() in metadata["str2idx"]
-                else metadata["str2idx"][UNKNOWN_SYMBOL]
+        def __replace_token_with_idx(value: Any, metadata: Dict[str, Any], fallback_symbol_idx: int) -> int:
+            stripped_value = value.strip()
+            if stripped_value in metadata["str2idx"]:
+                return metadata["str2idx"][stripped_value]
+            logger.warning(
+                f"""
+                Encountered unknown symbol '{stripped_value}' for '{column.name}' during category
+                feature preprocessing. This should never happen during training. If this happens during
+                inference, this may be an indication that not all possible symbols were present in your
+                training set. Consider re-splitting your data to ensure full representation, or setting
+                preprocessing.most_common parameter to be smaller than this feature's total vocabulary
+                size, {len(metadata["str2idx"])}, which will ensure that the model is architected and
+                trained with an UNKNOWN symbol. Returning the index for the most frequent symbol,
+                {metadata["idx2str"][fallback_symbol_idx]}, instead.
+                """
             )
-        ).astype(int_type(metadata["vocab_size"]))
+            return fallback_symbol_idx
+
+        # No unknown symbol in Metadata from preprocessing means that all values
+        # should be mappable to vocabulary
+        if UNKNOWN_SYMBOL not in metadata["str2idx"]:
+            # If no unknown is defined, just use the most popular token's index as the fallback index
+            most_popular_token = max(metadata["str2freq"], key=metadata["str2freq"].get)
+            most_popular_token_idx = metadata["str2idx"].get(most_popular_token)
+            return column.map(lambda x: __replace_token_with_idx(x, metadata, most_popular_token_idx)).astype(
+                int_type(metadata["vocab_size"])
+            )
+        else:
+            return column.map(
+                lambda x: (
+                    metadata["str2idx"][x.strip()]
+                    if x.strip() in metadata["str2idx"]
+                    else metadata["str2idx"][UNKNOWN_SYMBOL]
+                )
+            ).astype(int_type(metadata["vocab_size"]))
 
     @staticmethod
     def add_feature_data(
@@ -165,16 +190,16 @@ class CategoryFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
+@register_input_feature(CATEGORY)
 class CategoryInputFeature(CategoryFeatureMixin, InputFeature):
-    encoder = "dense"
+    def __init__(self, input_feature_config: Union[CategoryInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature)
-        self.overwrite_defaults(feature)
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs):
         assert isinstance(inputs, torch.Tensor)
@@ -209,29 +234,41 @@ class CategoryInputFeature(CategoryFeatureMixin, InputFeature):
 
     @staticmethod
     def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature["vocab"] = feature_metadata["idx2str"]
+        input_feature[ENCODER]["vocab"] = feature_metadata["idx2str"]
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
+        defaults = CategoryInputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+
+    @staticmethod
+    def get_schema_cls():
+        return CategoryInputFeatureConfig
 
     @staticmethod
     def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
         return _CategoryPreprocessing(metadata)
 
 
+@register_output_feature(CATEGORY)
 class CategoryOutputFeature(CategoryFeatureMixin, OutputFeature):
-    decoder = "classifier"
-    loss = {TYPE: SOFTMAX_CROSS_ENTROPY}
     metric_functions = {LOSS: None, ACCURACY: None, HITS_AT_K: None}
     default_validation_metric = ACCURACY
-    num_classes = 0
-    top_k = 3
 
-    def __init__(self, feature, output_features: Dict[str, OutputFeature]):
-        super().__init__(feature, output_features)
-        self.overwrite_defaults(feature)
-        self.decoder_obj = self.initialize_decoder(feature)
+    def __init__(
+        self,
+        output_feature_config: Union[CategoryOutputFeatureConfig, Dict],
+        output_features: Dict[str, OutputFeature],
+        **kwargs,
+    ):
+        output_feature_config = self.load_config(output_feature_config)
+        self.num_classes = output_feature_config.num_classes
+        self.top_k = output_feature_config.top_k
+        super().__init__(output_feature_config, output_features, **kwargs)
+        if hasattr(output_feature_config.decoder, "num_classes"):
+            output_feature_config.decoder.num_classes = output_feature_config.num_classes
+        self.decoder_obj = self.initialize_decoder(output_feature_config.decoder)
         self._setup_loss()
         self._setup_metrics()
 
@@ -243,15 +280,15 @@ class CategoryOutputFeature(CategoryFeatureMixin, OutputFeature):
         # hidden: shape [batch_size, size of final fully connected layer]
         return {LOGITS: self.decoder_obj(hidden), PROJECTION_INPUT: hidden}
 
-    def create_calibration_module(self, feature) -> torch.nn.Module:
+    def create_calibration_module(self, feature: CategoryOutputFeatureConfig) -> torch.nn.Module:
         """Creates the appropriate calibration module based on the feature config.
 
         Today, only one type of calibration ("temperature_scaling") is available, but more options may be supported in
         the future.
         """
-        if feature.get("calibration"):
+        if feature.calibration:
             calibration_cls = calibration.get_calibration_cls(CATEGORY, "temperature_scaling")
-            return calibration_cls(num_classes=feature["num_classes"])
+            return calibration_cls(num_classes=self.num_classes)
         return None
 
     def create_predict_module(self) -> PredictModule:
@@ -387,64 +424,61 @@ class CategoryOutputFeature(CategoryFeatureMixin, OutputFeature):
         self,
         predictions,
         metadata,
-        output_directory,
-        backend,
     ):
         predictions_col = f"{self.feature_name}_{PREDICTIONS}"
         if predictions_col in predictions:
             if "idx2str" in metadata:
-                predictions[predictions_col] = backend.df_engine.map_objects(
-                    predictions[predictions_col], lambda pred: metadata["idx2str"][pred]
-                )
+                predictions[predictions_col] = predictions[predictions_col].map(lambda pred: metadata["idx2str"][pred])
 
         probabilities_col = f"{self.feature_name}_{PROBABILITIES}"
         if probabilities_col in predictions:
             prob_col = f"{self.feature_name}_{PROBABILITY}"
             predictions[prob_col] = predictions[probabilities_col].map(max)
-            predictions[probabilities_col] = backend.df_engine.map_objects(
-                predictions[probabilities_col], lambda pred: pred.tolist()
-            )
+            predictions[probabilities_col] = predictions[probabilities_col].map(lambda pred: pred.tolist())
             if "idx2str" in metadata:
                 for i, label in enumerate(metadata["idx2str"]):
                     key = f"{probabilities_col}_{label}"
 
                     # Use default param to force a capture before the loop completes, see:
                     # https://stackoverflow.com/questions/2295290/what-do-lambda-function-closures-capture
-                    predictions[key] = backend.df_engine.map_objects(
-                        predictions[probabilities_col],
+                    predictions[key] = predictions[probabilities_col].map(
                         lambda prob, i=i: prob[i],
                     )
 
         top_k_col = f"{self.feature_name}_predictions_top_k"
         if top_k_col in predictions:
             if "idx2str" in metadata:
-                predictions[top_k_col] = backend.df_engine.map_objects(
-                    predictions[top_k_col], lambda pred_top_k: [metadata["idx2str"][pred] for pred in pred_top_k]
+                predictions[top_k_col] = predictions[top_k_col].map(
+                    lambda pred_top_k: [metadata["idx2str"][pred] for pred in pred_top_k]
                 )
 
         return predictions
 
     @staticmethod
     def populate_defaults(output_feature):
+        defaults = CategoryOutputFeatureConfig()
+
         # If Loss is not defined, set an empty dictionary
         set_default_value(output_feature, LOSS, {})
-
         # Populate the default values for LOSS if they aren't defined already
+        set_default_values(output_feature[LOSS], defaults.loss.Schema().dump(defaults.loss))
+
         set_default_values(
-            output_feature[LOSS],
+            output_feature,
             {
-                TYPE: "softmax_cross_entropy",
-                "class_weights": 1,
-                "robust_lambda": 0,
-                "confidence_penalty": 0,
-                "class_similarities_temperature": 0,
-                "weight": 1,
+                DECODER: {
+                    TYPE: defaults.decoder.type,
+                },
+                TOP_K: defaults.top_k,
+                DEPENDENCIES: defaults.dependencies,
+                REDUCE_INPUT: defaults.reduce_input,
+                REDUCE_DEPENDENCIES: defaults.reduce_dependencies,
             },
         )
 
-        set_default_values(
-            output_feature, {"top_k": 3, "dependencies": [], "reduce_input": SUM, "reduce_dependencies": SUM}
-        )
+    @staticmethod
+    def get_schema_cls():
+        return CategoryOutputFeatureConfig
 
     @staticmethod
     def create_postproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
