@@ -13,9 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+import copy
 import logging
 import random
-from typing import Any, Dict
+from typing import Any, Dict, Union
 
 import numpy as np
 import pandas as pd
@@ -23,26 +24,31 @@ import torch
 from torch import nn
 
 from ludwig.constants import (
+    CLIP,
     COLUMN,
-    FILL_WITH_CONST,
+    DECODER,
+    DEPENDENCIES,
+    ENCODER,
     HIDDEN,
     LOGITS,
     LOSS,
     MEAN_ABSOLUTE_ERROR,
     MEAN_SQUARED_ERROR,
-    MISSING_VALUE_STRATEGY_OPTIONS,
     NAME,
     NUMBER,
     PREDICTIONS,
     PROC_COLUMN,
     R2,
+    REDUCE_DEPENDENCIES,
+    REDUCE_INPUT,
     ROOT_MEAN_SQUARED_ERROR,
     ROOT_MEAN_SQUARED_PERCENTAGE_ERROR,
-    SUM,
     TIED,
     TYPE,
 )
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature, OutputFeature, PredictModule
+from ludwig.schema.features.number_feature import NumberInputFeatureConfig, NumberOutputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature, register_output_feature
 from ludwig.utils import output_feature_utils
 from ludwig.utils.misc_utils import get_from_registry, set_default_value, set_default_values
 from ludwig.utils.types import TorchscriptPreprocessingInput
@@ -55,6 +61,14 @@ class ZScoreTransformer(nn.Module):
         super().__init__()
         self.mu = float(mean) if mean is not None else mean
         self.sigma = float(std) if std is not None else std
+        self.feature_name = kwargs.get(NAME, "")
+        if self.sigma == 0:
+            raise RuntimeError(
+                f"Cannot apply zscore normalization to `{self.feature_name}` since it has a standard deviation of 0. "
+                f"This is most likely because `{self.feature_name}` has a constant value of {self.mu} for all rows in "
+                "the dataset. Consider removing this feature from your Ludwig config since it is not useful for "
+                "your machine learning model."
+            )
 
     def transform(self, x: np.ndarray) -> np.ndarray:
         return (x - self.mu) / self.sigma
@@ -195,8 +209,10 @@ class _NumberPostprocessing(torch.nn.Module):
         self.numeric_transformer = get_transformer(metadata, metadata["preprocessing"])
         self.predictions_key = PREDICTIONS
 
-    def forward(self, preds: Dict[str, torch.Tensor]) -> Dict[str, Any]:
-        return {self.predictions_key: self.numeric_transformer.inverse_transform_inference(preds[self.predictions_key])}
+    def forward(self, preds: Dict[str, torch.Tensor], feature_name: str) -> Dict[str, Any]:
+        predictions = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.predictions_key)
+
+        return {self.predictions_key: self.numeric_transformer.inverse_transform_inference(predictions)}
 
 
 class _NumberPredict(PredictModule):
@@ -222,26 +238,7 @@ class NumberFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "missing_value_strategy": FILL_WITH_CONST,
-            "fill_value": 0,
-            "normalization": None,
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "missing_value_strategy": {
-                "type": "string",
-                "enum": MISSING_VALUE_STRATEGY_OPTIONS,
-            },
-            "fill_value": {"type": "number"},
-            "computed_fill_value": {"type": "number"},
-            "normalization": {
-                "type": ["string", "null"],
-                "enum": list(numeric_transformation_registry.keys()),
-            },
-        }
+        return NumberInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -276,8 +273,11 @@ class NumberFeatureMixin(BaseFeatureMixin):
         #     return series
 
         def normalize(series: pd.Series) -> pd.Series:
+            _feature_metadata = copy.deepcopy(metadata[feature_config[NAME]])
+            _feature_metadata.update({NAME: feature_config[NAME]})
+
             # retrieve request numeric transformer
-            numeric_transformer = get_transformer(metadata[feature_config[NAME]], preprocessing_parameters)
+            numeric_transformer = get_transformer(_feature_metadata, preprocessing_parameters)
 
             # transform input numeric values with specified transformer
             transformed_values = numeric_transformer.transform(series.values)
@@ -293,18 +293,17 @@ class NumberFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
+@register_input_feature(NUMBER)
 class NumberInputFeature(NumberFeatureMixin, InputFeature):
-    encoder = "passthrough"
+    def __init__(self, input_feature_config: Union[NumberInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
+        input_feature_config.encoder.input_size = self.input_shape[-1]
 
-    def __init__(self, feature, encoder_obj=None):
-        # Required for certain encoders, maybe pass into initialize_encoder
-        super().__init__(feature)
-        self.overwrite_defaults(feature)
-        feature["input_size"] = self.input_shape[-1]
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs):
         assert isinstance(inputs, torch.Tensor)
@@ -335,7 +334,13 @@ class NumberInputFeature(NumberFeatureMixin, InputFeature):
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
+        defaults = NumberInputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+
+    @staticmethod
+    def get_schema_cls():
+        return NumberInputFeatureConfig
 
     @classmethod
     def get_preproc_input_dtype(cls, metadata: Dict[str, Any]) -> str:
@@ -346,9 +351,8 @@ class NumberInputFeature(NumberFeatureMixin, InputFeature):
         return _NumberPreprocessing(metadata)
 
 
+@register_output_feature(NUMBER)
 class NumberOutputFeature(NumberFeatureMixin, OutputFeature):
-    decoder = "regressor"
-    loss = {TYPE: MEAN_SQUARED_ERROR}
     metric_functions = {
         LOSS: None,
         MEAN_SQUARED_ERROR: None,
@@ -358,12 +362,17 @@ class NumberOutputFeature(NumberFeatureMixin, OutputFeature):
         R2: None,
     }
     default_validation_metric = MEAN_SQUARED_ERROR
-    clip = None
 
-    def __init__(self, feature, output_features: Dict[str, OutputFeature]):
-        super().__init__(feature, output_features)
-        self.overwrite_defaults(feature)
-        self.decoder_obj = self.initialize_decoder(feature)
+    def __init__(
+        self,
+        output_feature_config: Union[NumberOutputFeatureConfig, Dict],
+        output_features: Dict[str, OutputFeature],
+        **kwargs,
+    ):
+        output_feature_config = self.load_config(output_feature_config)
+        self.clip = output_feature_config.clip
+        super().__init__(output_feature_config, output_features, **kwargs)
+        self.decoder_obj = self.initialize_decoder(output_feature_config.decoder)
         self._setup_loss()
         self._setup_metrics()
 
@@ -372,19 +381,19 @@ class NumberOutputFeature(NumberFeatureMixin, OutputFeature):
         return self.decoder_obj(hidden)
 
     def create_predict_module(self) -> PredictModule:
-        if self.clip is not None and not (isinstance(self.clip, (list, tuple)) and len(self.clip) == 2):
+        if getattr(self, "clip", None) and not (isinstance(self.clip, (list, tuple)) and len(self.clip) == 2):
             raise ValueError(
                 f"The clip parameter of {self.feature_name} is {self.clip}. "
                 f"It must be a list or a tuple of length 2."
             )
-        return _NumberPredict(self.clip)
+        return _NumberPredict(getattr(self, "clip", None))
 
     def get_prediction_set(self):
         return {PREDICTIONS, LOGITS}
 
     @property
     def input_shape(self) -> torch.Size:
-        return torch.Size([self.input_size])
+        return torch.Size([self.decoder_obj.config.input_size])
 
     @classmethod
     def get_output_dtype(cls):
@@ -407,8 +416,6 @@ class NumberOutputFeature(NumberFeatureMixin, OutputFeature):
         self,
         predictions,
         metadata,
-        output_directory,
-        backend,
     ):
         predictions_col = f"{self.feature_name}_{PREDICTIONS}"
         if predictions_col in predictions:
@@ -417,36 +424,34 @@ class NumberOutputFeature(NumberFeatureMixin, OutputFeature):
                 metadata["preprocessing"].get("normalization", None),
                 numeric_transformation_registry,
             )(**metadata)
-            predictions[predictions_col] = backend.df_engine.map_objects(
-                predictions[predictions_col],
-                lambda pred: numeric_transformer.inverse_transform(pred),
+            predictions[predictions_col] = predictions[predictions_col].map(
+                lambda pred: numeric_transformer.inverse_transform(pred)
             )
 
         return predictions
 
-    @staticmethod
-    def postprocess_inference_graph(
-        preds: Dict[str, torch.Tensor], metadata: Dict[str, Any]
-    ) -> Dict[str, torch.Tensor]:
-        numeric_transformer = get_transformer(metadata, metadata["preprocessing"])
-        return {PREDICTIONS: numeric_transformer.inverse_transform_inference(preds[PREDICTIONS])}
-
     @staticmethod
     def populate_defaults(output_feature):
-        set_default_value(output_feature, LOSS, {TYPE: "mean_squared_error", "weight": 1})
-        set_default_value(output_feature[LOSS], TYPE, "mean_squared_error")
-        set_default_value(output_feature[LOSS], "weight", 1)
-
+        defaults = NumberOutputFeatureConfig()
+        set_default_value(output_feature, LOSS, {})
+        set_default_values(output_feature[LOSS], defaults.loss.Schema().dump(defaults.loss))
         set_default_values(
             output_feature,
             {
-                "clip": None,
-                "dependencies": [],
-                "reduce_input": SUM,
-                "reduce_dependencies": SUM,
+                DECODER: {
+                    TYPE: defaults.decoder.type,
+                },
+                CLIP: defaults.clip,
+                DEPENDENCIES: defaults.dependencies,
+                REDUCE_INPUT: defaults.reduce_input,
+                REDUCE_DEPENDENCIES: defaults.reduce_dependencies,
             },
         )
 
+    @staticmethod
+    def get_schema_cls():
+        return NumberOutputFeatureConfig
+
     @classmethod
     def get_postproc_output_dtype(cls, metadata: Dict[str, Any]) -> str:
         return "float32"
