@@ -16,49 +16,51 @@
 
 import logging
 from functools import partial
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Union
 
 import numpy as np
 import torch
 
 from ludwig.constants import (
     COLUMN,
+    DECODER,
+    DEPENDENCIES,
     EDIT_DISTANCE,
-    FILL_WITH_CONST,
+    ENCODER,
     LAST_ACCURACY,
     LAST_PREDICTIONS,
     LENGTHS,
     LOSS,
-    MISSING_VALUE_STRATEGY_OPTIONS,
     NAME,
     PERPLEXITY,
     PREDICTIONS,
     PROBABILITIES,
     PROBABILITY,
     PROC_COLUMN,
+    REDUCE_DEPENDENCIES,
+    REDUCE_INPUT,
     SEQUENCE,
     SEQUENCE_ACCURACY,
-    SUM,
     TIED,
     TOKEN_ACCURACY,
     TYPE,
 )
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature, OutputFeature, PredictModule
 from ludwig.features.feature_utils import compute_sequence_probability, compute_token_probabilities
+from ludwig.schema.features.sequence_feature import SequenceInputFeatureConfig, SequenceOutputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature, register_output_feature
 from ludwig.utils import output_feature_utils
 from ludwig.utils.math_utils import softmax
-from ludwig.utils.misc_utils import get_from_registry, set_default_value
+from ludwig.utils.misc_utils import get_from_registry, set_default_value, set_default_values
 from ludwig.utils.strings_utils import (
     build_sequence_matrix,
     create_vocabulary,
-    PADDING_SYMBOL,
     SpecialSymbol,
     START_SYMBOL,
     STOP_SYMBOL,
     tokenizer_registry,
     UNKNOWN_SYMBOL,
 )
-from ludwig.utils.tokenizers import TORCHSCRIPT_COMPATIBLE_TOKENIZERS
 from ludwig.utils.types import DataFrame, TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
@@ -69,16 +71,15 @@ class _SequencePreprocessing(torch.nn.Module):
 
     def __init__(self, metadata: Dict[str, Any]):
         super().__init__()
-        if metadata["preprocessing"]["tokenizer"] not in TORCHSCRIPT_COMPATIBLE_TOKENIZERS:
-            raise ValueError(
-                f"{metadata['preprocessing']['tokenizer']} is not supported by torchscript. Please use "
-                f"one of {TORCHSCRIPT_COMPATIBLE_TOKENIZERS}."
-            )
-
         self.lowercase = metadata["preprocessing"]["lowercase"]
-        self.tokenizer = get_from_registry(metadata["preprocessing"]["tokenizer"], tokenizer_registry)(
+        self.tokenizer_type = metadata["preprocessing"]["tokenizer"]
+        self.tokenizer = get_from_registry(self.tokenizer_type, tokenizer_registry)(
             pretrained_model_name_or_path=metadata["preprocessing"].get("pretrained_model_name_or_path", None)
         )
+
+        if not isinstance(self.tokenizer, torch.nn.Module):
+            raise ValueError(f"tokenizer must be a torch.nn.Module, got {self.tokenizer}")
+
         self.padding_symbol = metadata["preprocessing"]["padding_symbol"]
         self.unknown_symbol = metadata["preprocessing"]["unknown_symbol"]
         self.start_symbol = START_SYMBOL
@@ -92,38 +93,58 @@ class _SequencePreprocessing(torch.nn.Module):
         if not torch.jit.isinstance(v, List[str]):
             raise ValueError(f"Unsupported input: {v}")
 
-        v = [self.computed_fill_value if s == "nan" else s for s in v]
+        futures: List[torch.jit.Future[torch.Tensor]] = []
+        for sequence in v:
+            futures.append(
+                torch.jit.fork(
+                    self._process_sequence,
+                    sequence,
+                )
+            )
 
-        if self.lowercase:
-            sequences = [sequence.lower() for sequence in v]
+        sequence_matrix = []
+        for future in futures:
+            sequence_matrix.append(torch.jit.wait(future))
+
+        return torch.stack(sequence_matrix)
+
+    def _process_sequence(self, sequence: str) -> torch.Tensor:
+        sequence = self.computed_fill_value if sequence == "nan" else sequence
+
+        # If tokenizer is HF, we defer lowercase transformation to the tokenizer.
+        if self.lowercase and self.tokenizer_type != "hf_tokenizer":
+            sequence_str: str = sequence.lower()
         else:
-            sequences = v
+            sequence_str: str = sequence
 
-        unit_sequences = self.tokenizer(sequences)
-        # refines type of unit_sequences from Any to List[List[str]]
-        assert torch.jit.isinstance(unit_sequences, List[List[str]]), "unit_sequences is not a list of lists."
+        sequence_vector = torch.full([self.max_sequence_length], self.unit_to_id[self.padding_symbol])
 
-        sequence_matrix = torch.full(
-            [len(unit_sequences), self.max_sequence_length], self.unit_to_id[self.padding_symbol]
-        )
-        sequence_matrix[:, 0] = self.unit_to_id[self.start_symbol]
-        for sample_idx, unit_sequence in enumerate(unit_sequences):
-            # Add <EOS> if sequence length is less than max_sequence_length. Else, truncate to max_sequence_length.
-            if len(unit_sequence) + 1 < self.max_sequence_length:
-                sequence_length = len(unit_sequence)
-                sequence_matrix[sample_idx][len(unit_sequence) + 1] = self.unit_to_id[self.stop_symbol]
+        if self.tokenizer_type == "hf_tokenizer":
+            unit_sequence = self.tokenizer(sequence)
+            assert torch.jit.isinstance(unit_sequence, List[int])
+            # Handles start, stop, and unknown symbols implicitly
+            sequence_vector[: len(unit_sequence)] = torch.tensor(unit_sequence)
+            return sequence_vector
+
+        # If tokenizer is not HF, we manually convert tokens to IDs and insert start, stop, and unknown symbols.
+        unit_sequence = self.tokenizer(sequence_str)
+        assert torch.jit.isinstance(unit_sequence, List[str])
+
+        sequence_vector[0] = self.unit_to_id[self.start_symbol]
+        if len(unit_sequence) + 1 < self.max_sequence_length:
+            sequence_length = len(unit_sequence)
+            sequence_vector[len(unit_sequence) + 1] = self.unit_to_id[self.stop_symbol]
+        else:
+            sequence_length = self.max_sequence_length - 1
+
+        for i in range(sequence_length):
+            curr_unit = unit_sequence[i]
+            if curr_unit in self.unit_to_id:
+                curr_id = self.unit_to_id[curr_unit]
             else:
-                sequence_length = self.max_sequence_length - 1
-
-            for i in range(sequence_length):
-                curr_unit = unit_sequence[i]
-                if curr_unit in self.unit_to_id:
-                    curr_id = self.unit_to_id[curr_unit]
-                else:
-                    curr_id = self.unit_to_id[self.unknown_symbol]
-                sequence_matrix[sample_idx][i + 1] = curr_id
-
-        return sequence_matrix
+                curr_id = self.unit_to_id[self.unknown_symbol]
+            sequence_vector[i + 1] = curr_id
+        return sequence_vector
 
 
 class _SequencePostprocessing(torch.nn.Module):
@@ -136,9 +157,10 @@ class _SequencePostprocessing(torch.nn.Module):
         self.probabilities_key = PROBABILITIES
         self.probability_key = PROBABILITY
 
-    def forward(self, preds: Dict[str, torch.Tensor]) -> Dict[str, Any]:
-        """Takes a dictionary of tensors and returns a dictionary of tensors."""
-        pred_predictions = preds[self.predictions_key]
+    def forward(self, preds: Dict[str, torch.Tensor], feature_name: str) -> Dict[str, Any]:
+        pred_predictions = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.predictions_key)
+        pred_probabilities = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.probabilities_key)
+
         predictions: List[List[str]] = []
         for sequence in pred_predictions:
             sequence_predictions: List[str] = []
@@ -151,7 +173,6 @@ class _SequencePostprocessing(torch.nn.Module):
                 sequence_predictions.append(unit_prediction)
             predictions.append(sequence_predictions)
 
-        pred_probabilities = preds[self.probabilities_key]
         probabilities, _ = torch.max(pred_probabilities, dim=-1)
         probability = torch.sum(torch.log(probabilities), dim=-1)
 
@@ -181,34 +202,7 @@ class SequenceFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "max_sequence_length": 256,
-            "most_common": 20000,
-            "padding_symbol": PADDING_SYMBOL,
-            "unknown_symbol": UNKNOWN_SYMBOL,
-            "padding": "right",
-            "tokenizer": "space",
-            "lowercase": False,
-            "vocab_file": None,
-            "missing_value_strategy": FILL_WITH_CONST,
-            "fill_value": UNKNOWN_SYMBOL,
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "max_sequence_length": {"type": "integer", "minimum": 0},
-            "most_common": {"type": "integer", "minimum": 0},
-            "padding_symbol": {"type": "string"},
-            "unknown_symbol": {"type": "string"},
-            "padding": {"type": "string", "enum": ["right", "left"]},
-            "tokenizer": {"type": "string", "enum": sorted(list(tokenizer_registry.keys()))},
-            "lowercase": {"type": "boolean"},
-            "vocab_file": {"type": ["string", "null"]},
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "fill_value": {"type": "string"},
-            "computed_fill_value": {"type": "string"},
-        }
+        return SequenceInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -265,20 +259,16 @@ class SequenceFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
+@register_input_feature(SEQUENCE)
 class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
-    encoder = "embed"
-    max_sequence_length = None
+    def __init__(self, input_feature_config: Union[SequenceInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature)
-        # TODO: Potentially abstract this feature-specific attribute overwrite to a consolidated design.
-        if "vocab" in feature:
-            feature["vocab_size"] = len(feature["vocab"])
-        self.overwrite_defaults(feature)
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs: torch.Tensor, mask=None):
         assert isinstance(inputs, torch.Tensor)
@@ -297,17 +287,23 @@ class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
 
     @staticmethod
     def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature["vocab"] = feature_metadata["idx2str"]
-        input_feature["max_sequence_length"] = feature_metadata["max_sequence_length"]
+        input_feature[ENCODER]["vocab"] = feature_metadata["idx2str"]
+        input_feature[ENCODER]["vocab_size"] = len(feature_metadata["idx2str"])
+        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
-        set_default_value(input_feature, "encoder", "parallel_cnn")
+        defaults = SequenceInputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+
+    @staticmethod
+    def get_schema_cls():
+        return SequenceInputFeatureConfig
 
     @property
     def input_shape(self) -> torch.Size:
-        return torch.Size([self.max_sequence_length])
+        return torch.Size([self.encoder_obj.config.max_sequence_length])
 
     @property
     def output_shape(self) -> torch.Size:
@@ -318,9 +314,8 @@ class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
         return _SequencePreprocessing(metadata)
 
 
+@register_output_feature(SEQUENCE)
 class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
-    decoder = "generator"
-    loss = {TYPE: "sequence_softmax_cross_entropy"}
     metric_functions = {
         LOSS: None,
         TOKEN_ACCURACY: None,
@@ -330,13 +325,16 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
         EDIT_DISTANCE: None,
     }
     default_validation_metric = LOSS
-    max_sequence_length = 0
-    num_classes = 0
 
-    def __init__(self, feature: Dict[str, Any], output_features: Dict[str, OutputFeature]):
-        super().__init__(feature, output_features)
-        self.overwrite_defaults(feature)
-        self.decoder_obj = self.initialize_decoder(feature)
+    def __init__(
+        self,
+        output_feature_config: Union[SequenceOutputFeatureConfig, Dict],
+        output_features: Dict[str, OutputFeature],
+        **kwargs,
+    ):
+        output_feature_config = self.load_config(output_feature_config)
+        super().__init__(output_feature_config, output_features, **kwargs)
+        self.decoder_obj = self.initialize_decoder(output_feature_config.decoder)
         self._setup_loss()
         self._setup_metrics()
 
@@ -360,14 +358,14 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
 
     @property
     def output_shape(self) -> torch.Size:
-        return torch.Size([self.max_sequence_length])
+        return torch.Size([self.decoder_obj.config.max_sequence_length])
 
     @staticmethod
     def update_config_with_metadata(output_feature, feature_metadata, *args, **kwargs):
-        output_feature["vocab_size"] = feature_metadata["vocab_size"]
-        output_feature["max_sequence_length"] = feature_metadata["max_sequence_length"]
+        output_feature[DECODER]["vocab_size"] = feature_metadata["vocab_size"]
+        output_feature[DECODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
         if isinstance(output_feature[LOSS]["class_weights"], (list, tuple)):
-            if len(output_feature[LOSS]["class_weights"]) != output_feature["vocab_size"]:
+            if len(output_feature[LOSS]["class_weights"]) != output_feature[DECODER]["vocab_size"]:
                 raise ValueError(
                     "The length of class_weights ({}) is not compatible with "
                     "the number of classes ({}) for feature {}. "
@@ -375,7 +373,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                     "and their order and consider there needs to be a weight "
                     "for the <UNK> and <PAD> class too.".format(
                         len(output_feature[LOSS]["class_weights"]),
-                        output_feature["vocab_size"],
+                        output_feature[DECODER]["vocab_size"],
                         output_feature[COLUMN],
                     )
                 )
@@ -417,14 +415,14 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                         )
                     )
 
-                if all_rows_length != output_feature["vocab_size"]:
+                if all_rows_length != output_feature[DECODER]["vocab_size"]:
                     raise ValueError(
                         "The size of the class_similarities matrix of {} is "
                         "{}, different from the number of classes ({}). "
                         "Check the metadata JSON file to see the classes "
                         "and their order and "
                         "consider <UNK> and <PAD> class too.".format(
-                            output_feature[COLUMN], all_rows_length, output_feature["vocab_size"]
+                            output_feature[COLUMN], all_rows_length, output_feature[DECODER]["vocab_size"]
                         )
                     )
 
@@ -449,8 +447,6 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
         self,
         result,
         metadata,
-        output_directory,
-        backend,
     ):
         predictions_col = f"{self.feature_name}_{PREDICTIONS}"
         lengths_col = f"{self.feature_name}_{LENGTHS}"
@@ -465,7 +461,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                         for token in [pred[i] for i in range(length)]
                     ]
 
-                result[predictions_col] = backend.df_engine.apply_objects(result, idx2str)
+                result[predictions_col] = result.apply(idx2str, axis=1)
 
         last_preds_col = f"{self.feature_name}_{LAST_PREDICTIONS}"
         if last_preds_col in result:
@@ -476,7 +472,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                         return metadata["idx2str"][last_pred]
                     return UNKNOWN_SYMBOL
 
-                result[last_preds_col] = backend.df_engine.map_objects(result[last_preds_col], last_idx2str)
+                result[last_preds_col] = result[last_preds_col].map(last_idx2str)
 
         probs_col = f"{self.feature_name}_{PROBABILITIES}"
         prob_col = f"{self.feature_name}_{PROBABILITY}"
@@ -484,14 +480,13 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
             # currently does not return full probabilties because usually it is huge:
             # dataset x length x classes
             # TODO: add a mechanism for letting the user decide to save it
-            result[probs_col] = backend.df_engine.map_objects(result[probs_col], compute_token_probabilities)
-            result[prob_col] = backend.df_engine.map_objects(
-                result[probs_col],
+            result[probs_col] = result[probs_col].map(compute_token_probabilities)
+            result[prob_col] = result[probs_col].map(
                 partial(
                     compute_sequence_probability,
                     max_sequence_length=metadata["max_sequence_length"],
                     return_log_prob=True,
-                ),
+                )
             )
 
         if lengths_col in result:
@@ -501,33 +496,33 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
 
     @staticmethod
     def populate_defaults(output_feature):
-        set_default_value(
-            output_feature,
-            LOSS,
-            {
-                TYPE: "sequence_softmax_cross_entropy",
-                "class_weights": 1,
-                "robust_lambda": 0,
-                "confidence_penalty": 0,
-                "class_similarities_temperature": 0,
-                "weight": 1,
-            },
-        )
+        defaults = SequenceOutputFeatureConfig()
+        set_default_value(output_feature, LOSS, {})
+        set_default_values(output_feature[LOSS], defaults.loss.Schema().dump(defaults.loss))
 
-        set_default_value(output_feature[LOSS], "unique", False)
-        set_default_value(output_feature, "decoder", "generator")
-
-        if output_feature["decoder"] == "tagger":
+        if DECODER in output_feature and TYPE in output_feature[DECODER] and output_feature[DECODER][TYPE] == "tagger":
             set_default_value(output_feature, "reduce_input", None)
 
-        set_default_value(output_feature, "dependencies", [])
-        set_default_value(output_feature, "reduce_input", SUM)
-        set_default_value(output_feature, "reduce_dependencies", SUM)
+        set_default_values(
+            output_feature,
+            {
+                DECODER: {
+                    TYPE: defaults.decoder.type,
+                },
+                DEPENDENCIES: defaults.dependencies,
+                REDUCE_INPUT: defaults.reduce_input,
+                REDUCE_DEPENDENCIES: defaults.reduce_dependencies,
+            },
+        )
 
     @staticmethod
     def create_postproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
         return _SequencePostprocessing(metadata)
 
+    @staticmethod
+    def get_schema_cls():
+        return SequenceOutputFeatureConfig
+
     def flatten(self, df: DataFrame) -> DataFrame:
         probs_col = f"{self.feature_name}_{PROBABILITIES}"
         df[probs_col] = df[probs_col].apply(lambda x: x.flatten())
@@ -535,5 +530,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
 
     def unflatten(self, df: DataFrame) -> DataFrame:
         probs_col = f"{self.feature_name}_{PROBABILITIES}"
-        df[probs_col] = df[probs_col].apply(lambda x: x.reshape(-1, self.num_classes), meta=(probs_col, "object"))
+        df[probs_col] = df[probs_col].apply(
+            lambda x: x.reshape(-1, self.decoder_obj.config.num_classes), meta=(probs_col, "object")
+        )
         return df
