@@ -1,9 +1,8 @@
 import copy
 import logging
 import os
-import warnings
 from pprint import pformat
-from typing import Any, Dict, List, Optional, Union
+from typing import List, Optional, Union
 
 import pandas as pd
 import yaml
@@ -13,19 +12,20 @@ from ludwig.api import LudwigModel
 from ludwig.backend import Backend, initialize_backend, LocalBackend
 from ludwig.callbacks import Callback
 from ludwig.constants import (
+    AUTO,
     COMBINED,
     EXECUTOR,
     GOAL,
-    GRID_SEARCH,
     HYPEROPT,
     LOSS,
+    MAX_CONCURRENT_TRIALS,
     METRIC,
-    MINIMIZE,
     NAME,
+    NUM_SAMPLES,
     OUTPUT_FEATURES,
+    PARAMETERS,
     PREPROCESSING,
-    RAY,
-    SPACE,
+    SEARCH_ALG,
     SPLIT,
     TEST,
     TRAINING,
@@ -35,12 +35,18 @@ from ludwig.constants import (
 from ludwig.data.split import get_splitter
 from ludwig.features.feature_registries import output_type_registry
 from ludwig.hyperopt.results import HyperoptResults
-from ludwig.hyperopt.utils import print_hyperopt_results, save_hyperopt_stats, should_tune_preprocessing
+from ludwig.hyperopt.utils import (
+    log_warning_if_all_grid_type_parameters,
+    print_hyperopt_results,
+    save_hyperopt_stats,
+    should_tune_preprocessing,
+    update_hyperopt_params_with_defaults,
+)
 from ludwig.utils.backward_compatibility import upgrade_to_latest_version
 from ludwig.utils.dataset_utils import generate_dataset_statistics
 from ludwig.utils.defaults import default_random_seed, merge_with_defaults
 from ludwig.utils.fs_utils import makedirs, open_file
-from ludwig.utils.misc_utils import get_class_attributes, get_from_registry, set_default_value, set_default_values
+from ludwig.utils.misc_utils import get_from_registry
 
 try:
     from ludwig.backend.ray import RayBackend
@@ -76,7 +82,7 @@ def hyperopt(
     skip_save_hyperopt_statistics: bool = False,
     output_directory: str = "results",
     gpus: Union[str, int, List[int]] = None,
-    gpu_memory_limit: int = None,
+    gpu_memory_limit: Optional[float] = None,
     allow_parallel_threads: bool = True,
     callbacks: List[Callback] = None,
     backend: Union[Backend, str] = None,
@@ -167,8 +173,8 @@ def hyperopt(
         model and the training progress files.
     :param gpus: (list, default: `None`) list of GPUs that are available
         for training.
-    :param gpu_memory_limit: (int, default: `None`) maximum memory in MB to
-        allocate per GPU device.
+    :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
+        [0, 1] allowed to allocate per GPU device.
     :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
         to use multithreading parallelism to improve performance at
         the cost of determinism.
@@ -211,23 +217,33 @@ def hyperopt(
 
     hyperopt_config = config[HYPEROPT]
 
+    # Explicitly default to a local backend to avoid picking up Ray or Horovod
+    # backend from the environment.
+    backend = backend or config_dict.get("backend") or "local"
+    backend = initialize_backend(backend)
+
     update_hyperopt_params_with_defaults(hyperopt_config)
 
+    # Check if all features are grid type parameters and log UserWarning if needed
+    log_warning_if_all_grid_type_parameters(hyperopt_config[PARAMETERS], hyperopt_config[EXECUTOR].get(NUM_SAMPLES))
+
+    # Infer max concurrent trials
+    if hyperopt_config[EXECUTOR].get(MAX_CONCURRENT_TRIALS) == AUTO:
+        hyperopt_config[EXECUTOR][MAX_CONCURRENT_TRIALS] = backend.max_concurrent_trials(hyperopt_config)
+        logger.info(f"Set max_concurrent_trials to {hyperopt_config[EXECUTOR][MAX_CONCURRENT_TRIALS]}")
+
     # Print hyperopt config
-    logger.info("Hyperopt config")
+    logger.info("Hyperopt Config")
     logger.info(pformat(hyperopt_config, indent=4))
     logger.info("\n")
 
-    search_alg = hyperopt_config["search_alg"]
+    search_alg = hyperopt_config[SEARCH_ALG]
     executor = hyperopt_config[EXECUTOR]
-    parameters = hyperopt_config["parameters"]
-    split = hyperopt_config["split"]
+    parameters = hyperopt_config[PARAMETERS]
+    split = hyperopt_config[SPLIT]
     output_feature = hyperopt_config["output_feature"]
-    metric = hyperopt_config["metric"]
-    goal = hyperopt_config["goal"]
-
-    # Check if all features are grid type parameters and log UserWarning if needed
-    log_warning_if_all_grid_type_parameters(parameters, executor.get("num_samples"))
+    metric = hyperopt_config[METRIC]
+    goal = hyperopt_config[GOAL]
 
     ######################
     # check validity of output_feature / metric/ split combination
@@ -294,10 +310,6 @@ def hyperopt(
         parameters, output_feature, metric, goal, split, search_alg=search_alg, **executor
     )
 
-    # Explicitly default to a local backend to avoid picking up Ray or Horovod
-    # backend from the environment.
-    backend = backend or config_dict.get("backend") or "local"
-    backend = initialize_backend(backend)
     if not (
         isinstance(backend, LocalBackend)
         or (isinstance(hyperopt_executor, RayTuneExecutor) and isinstance(backend, RayBackend))
@@ -381,16 +393,17 @@ def hyperopt(
         print_hyperopt_results(hyperopt_results)
 
         if not skip_save_hyperopt_statistics:
-            results_directory = os.path.join(output_directory, experiment_name)
-            makedirs(results_directory, exist_ok=True)
+            with backend.storage.artifacts.use_credentials():
+                results_directory = os.path.join(output_directory, experiment_name)
+                makedirs(results_directory, exist_ok=True)
 
-            hyperopt_stats = {
-                "hyperopt_config": hyperopt_config,
-                "hyperopt_results": [t.to_dict() for t in hyperopt_results.ordered_trials],
-            }
+                hyperopt_stats = {
+                    "hyperopt_config": hyperopt_config,
+                    "hyperopt_results": [t.to_dict() for t in hyperopt_results.ordered_trials],
+                }
 
-            save_hyperopt_stats(hyperopt_stats, results_directory)
-            logger.info(f"Hyperopt stats saved to: {results_directory}")
+                save_hyperopt_stats(hyperopt_stats, results_directory)
+                logger.info(f"Hyperopt stats saved to: {results_directory}")
 
     for callback in callbacks or []:
         callback.on_hyperopt_end(experiment_name)
@@ -399,43 +412,3 @@ def hyperopt(
     logger.info("Finished hyperopt")
 
     return hyperopt_results
-
-
-def log_warning_if_all_grid_type_parameters(hyperopt_parameter_config: Dict[str, Any], num_samples: int = 1) -> None:
-    """Logs warning if all parameters have a grid type search space and num_samples > 1 since this will result in
-    duplicate trials being created."""
-    if num_samples == 1:
-        return
-
-    total_grid_search_trials = 1
-
-    for _, param_info in hyperopt_parameter_config.items():
-        if param_info.get(SPACE, None) != GRID_SEARCH:
-            return
-        total_grid_search_trials *= len(param_info.get("values", []))
-
-    num_duplicate_trials = (total_grid_search_trials * num_samples) - total_grid_search_trials
-    warnings.warn(
-        "All hyperopt parameters in Ludwig config are using grid_search space, but number of samples "
-        f"({num_samples}) is greater than 1. This will result in {num_duplicate_trials} duplicate trials being "
-        "created. Consider setting `num_samples` to 1 in the hyperopt executor to prevent trial duplication.",
-        RuntimeWarning,
-    )
-
-
-def update_hyperopt_params_with_defaults(hyperopt_params):
-    from ludwig.hyperopt.execution import executor_registry
-
-    set_default_value(hyperopt_params, EXECUTOR, {})
-    set_default_value(hyperopt_params, SPLIT, VALIDATION)
-    set_default_value(hyperopt_params, "output_feature", COMBINED)
-    set_default_value(hyperopt_params, METRIC, LOSS)
-    set_default_value(hyperopt_params, GOAL, MINIMIZE)
-
-    set_default_values(hyperopt_params[EXECUTOR], {TYPE: RAY, "num_samples": 1})
-    executor = get_from_registry(hyperopt_params[EXECUTOR][TYPE], executor_registry)
-    executor_defaults = {k: v for k, v in executor.__dict__.items() if k in get_class_attributes(executor)}
-    set_default_values(
-        hyperopt_params[EXECUTOR],
-        executor_defaults,
-    )
