@@ -143,6 +143,7 @@ class Trainer(BaseTrainer):
         self.decay_steps = config.decay_steps
         self.staircase = config.staircase
         self.batch_size = config.batch_size
+        self.max_batch_size = config.max_batch_size
         self.eval_batch_size = config.batch_size if config.eval_batch_size is None else config.eval_batch_size
         self.should_shuffle = config.should_shuffle
         self._validation_field = config.validation_field
@@ -159,7 +160,6 @@ class Trainer(BaseTrainer):
         self.increase_batch_size_on_plateau = config.increase_batch_size_on_plateau
         self.increase_batch_size_on_plateau_patience = config.increase_batch_size_on_plateau_patience
         self.increase_batch_size_on_plateau_rate = config.increase_batch_size_on_plateau_rate
-        self.increase_batch_size_on_plateau_max = config.increase_batch_size_on_plateau_max
         self.increase_batch_size_eval_metric = config.increase_batch_size_eval_metric
         self.increase_batch_size_eval_split = config.increase_batch_size_eval_split
         self.learning_rate_warmup_epochs = config.learning_rate_warmup_epochs
@@ -457,7 +457,15 @@ class Trainer(BaseTrainer):
 
             def _is_valid_batch_size(batch_size):
                 # make sure that batch size is valid (e.g. less than size of ds)
-                return batch_size < len(training_set)
+                is_smaller_than_training_set = batch_size < len(training_set)
+                is_under_max_batch_size = batch_size <= self.max_batch_size
+                is_valid = is_smaller_than_training_set and is_under_max_batch_size
+                if not is_valid:
+                    logger.info(
+                        f"Batch size {batch_size} is invalid, must be smaller than training set size "
+                        f"{len(training_set)} and less than or equal to max batch size {self.max_batch_size}"
+                    )
+                return is_valid
 
             # TODO (ASN) : Circle back on how we want to set default placeholder value
             # Currently, since self.batch_size is originally set to auto, we provide a
@@ -629,7 +637,7 @@ class Trainer(BaseTrainer):
                 self.increase_batch_size_on_plateau,
                 self.increase_batch_size_on_plateau_patience,
                 self.increase_batch_size_on_plateau_rate,
-                self.increase_batch_size_on_plateau_max,
+                self.max_batch_size,
                 self.increase_batch_size_eval_metric,
                 self.increase_batch_size_eval_split,
                 early_stopping_steps,
@@ -1327,7 +1335,7 @@ class Trainer(BaseTrainer):
                     )
                 ):
                     progress_tracker.batch_size = min(
-                        (increase_batch_size_on_plateau_rate * progress_tracker.batch_size),
+                        int(increase_batch_size_on_plateau_rate * progress_tracker.batch_size),
                         increase_batch_size_on_plateau_max,
                     )
 
