@@ -351,18 +351,22 @@ class LightGBMTrainer(BaseTrainer):
     ) -> bool:
         self.callback(lambda c: c.on_batch_start(self, progress_tracker, save_path))
 
-        booster = None
         evals_result = {}
-        booster = self.train_step(
-            params, lgb_train, eval_sets, eval_names, booster, self.boosting_rounds_per_checkpoint, evals_result
+        self.model.lgbm_model = self.train_step(
+            params,
+            lgb_train,
+            eval_sets,
+            eval_names,
+            self.model.lgbm_model,
+            self.boosting_rounds_per_checkpoint,
+            evals_result,
         )
 
         progress_bar.update(self.boosting_rounds_per_checkpoint)
         progress_tracker.steps += self.boosting_rounds_per_checkpoint
-        progress_tracker.last_improvement_steps = booster.best_iteration
+        progress_tracker.last_improvement_steps = self.model.lgbm_model.best_iteration_
 
         # convert to pytorch for inference
-        self.model.lgb_booster = booster
         self.model.compile()
         self.model = self.model.to(self.device)
 
@@ -463,10 +467,10 @@ class LightGBMTrainer(BaseTrainer):
         lgb_train: lgb.Dataset,
         eval_sets: List[lgb.Dataset],
         eval_names: List[str],
-        booster: lgb.Booster,
+        init_model: lgb.LGBMModel,
         boost_rounds_per_train_step: int,
         evals_result: Dict,
-    ) -> lgb.Booster:
+    ) -> lgb.LGBMModel:
         """Trains a LightGBM model.
 
         Args:
@@ -478,18 +482,21 @@ class LightGBMTrainer(BaseTrainer):
         Returns:
             LightGBM Booster model
         """
-        gbm = lgb.train(
-            params,
-            lgb_train,
-            init_model=booster,
-            num_boost_round=boost_rounds_per_train_step,
-            valid_sets=eval_sets,
-            valid_names=eval_names,
-            feature_name=list(self.model.input_features.keys()),
+        output_feature = next(iter(self.model.output_features.values()))
+        gbm_sklearn_cls = lgb.LGBMRegressor if output_feature.type() == NUMBER else lgb.LGBMClassifier
+
+        gbm = gbm_sklearn_cls(n_estimators=boost_rounds_per_train_step, **params).fit(
+            X=lgb_train.get_data(),
+            y=lgb_train.get_label(),
+            init_model=init_model,
+            eval_set=[(ds.get_data(), ds.get_label()) for ds in eval_sets],
+            eval_names=eval_names,
+            # add early stopping callback to populate best_iteration
+            callbacks=[lgb.early_stopping(boost_rounds_per_train_step)],
             # NOTE: hummingbird does not support categorical features
             # categorical_feature=categorical_features,
-            evals_result=evals_result,
         )
+        evals_result.update(gbm.evals_result_)
 
         return gbm
 
@@ -514,10 +521,6 @@ class LightGBMTrainer(BaseTrainer):
 
         # TODO: construct new datasets by running encoders (for text, image)
 
-        # TODO: only single task currently
-        if len(output_features) > 1:
-            raise ValueError("Only single task currently supported")
-
         metrics_names = get_metric_names(output_features)
 
         # check if validation_field is valid
@@ -719,17 +722,17 @@ class LightGBMTrainer(BaseTrainer):
     def _construct_lgb_params(self) -> Tuple[dict, dict]:
         output_params = {}
         feature = next(iter(self.model.output_features.values()))
-        if feature.type() == CATEGORY:
+        if feature.type() == BINARY or (hasattr(feature, "num_classes") and feature.num_classes == 2):
+            output_params = {
+                "objective": "binary",
+                "metric": ["binary_logloss"],
+            }
+        elif feature.type() == CATEGORY:
             output_params = {
                 "objective": "multiclass",
                 "metric": ["multi_logloss"],
                 "num_class": feature.num_classes,
             }
-        elif feature.type() == BINARY:
-            output_params = {
-                "objective": "binary",
-                "metric": ["binary_logloss"],
-            }
         elif feature.type() == NUMBER:
             output_params = {
                 "objective": "regression",
@@ -799,15 +802,15 @@ class LightGBMTrainer(BaseTrainer):
         y_train = training_set.to_df(self.model.output_features.values())
 
         # create dataset for lightgbm
-        # if you want to re-use data, remember to set free_raw_data=False
-        lgb_train = lgb.Dataset(X_train, label=y_train)
+        # keep raw data for continued training https://github.com/microsoft/LightGBM/issues/4965#issuecomment-1019344293
+        lgb_train = lgb.Dataset(X_train, label=y_train, free_raw_data=False).construct()
 
         eval_sets = [lgb_train]
         eval_names = [LightGBMTrainer.TRAIN_KEY]
         if validation_set is not None:
             X_val = validation_set.to_df(self.model.input_features.values())
             y_val = validation_set.to_df(self.model.output_features.values())
-            lgb_val = lgb.Dataset(X_val, label=y_val, reference=lgb_train)
+            lgb_val = lgb.Dataset(X_val, label=y_val, reference=lgb_train, free_raw_data=False).construct()
             eval_sets.append(lgb_val)
             eval_names.append(LightGBMTrainer.VALID_KEY)
         else:
@@ -817,7 +820,7 @@ class LightGBMTrainer(BaseTrainer):
         if test_set is not None:
             X_test = test_set.to_df(self.model.input_features.values())
             y_test = test_set.to_df(self.model.output_features.values())
-            lgb_test = lgb.Dataset(X_test, label=y_test, reference=lgb_train)
+            lgb_test = lgb.Dataset(X_test, label=y_test, reference=lgb_train, free_raw_data=False).construct()
             eval_sets.append(lgb_test)
             eval_names.append(LightGBMTrainer.TEST_KEY)
 
@@ -898,10 +901,10 @@ class LightGBMRayTrainer(LightGBMTrainer):
         lgb_train: "RayDMatrix",  # noqa: F821
         eval_sets: List["RayDMatrix"],  # noqa: F821
         eval_names: List[str],
-        booster: lgb.Booster,
+        init_model: lgb.LGBMModel,
         boost_rounds_per_train_step: int,
         evals_result: Dict,
-    ) -> lgb.Booster:
+    ) -> lgb.LGBMModel:
         """Trains a LightGBM model using ray.
 
         Args:
@@ -913,23 +916,26 @@ class LightGBMRayTrainer(LightGBMTrainer):
         Returns:
             LightGBM Booster model
         """
-        from lightgbm_ray import train as lgb_ray_train
+        from lightgbm_ray import RayLGBMClassifier, RayLGBMRegressor
 
-        gbm = lgb_ray_train(
-            params,
-            lgb_train,
-            init_model=booster,
-            num_boost_round=boost_rounds_per_train_step,
-            valid_sets=eval_sets,
-            valid_names=eval_names,
-            feature_name=list(self.model.input_features.keys()),
-            evals_result=evals_result,
+        output_feature = next(iter(self.model.output_features.values()))
+        gbm_sklearn_cls = RayLGBMRegressor if output_feature.type() == NUMBER else RayLGBMClassifier
+
+        gbm = gbm_sklearn_cls(n_estimators=boost_rounds_per_train_step, **params).fit(
+            X=lgb_train,
+            y=None,
+            init_model=init_model,
+            eval_set=[(s, n) for s, n in zip(eval_sets, eval_names)],
+            eval_names=eval_names,
+            # add early stopping callback to populate best_iteration
+            callbacks=[lgb.early_stopping(boost_rounds_per_train_step)],
+            ray_params=_map_to_lgb_ray_params(self.trainer_kwargs),
             # NOTE: hummingbird does not support categorical features
             # categorical_feature=categorical_features,
-            ray_params=_map_to_lgb_ray_params(self.trainer_kwargs),
         )
+        evals_result.update(gbm.evals_result_)
 
-        return gbm.booster_
+        return gbm.to_local()
 
     def _construct_lgb_datasets(
         self,
