@@ -13,11 +13,20 @@ from sklearn.model_selection import train_test_split
 from ludwig import globals as global_vars
 from ludwig.api import LudwigModel
 from ludwig.backend import LOCAL_BACKEND
-from ludwig.constants import TRAINER, TRAINING
+from ludwig.constants import (
+    CATEGORY,
+    DEFAULTS,
+    EPOCHS,
+    INPUT_FEATURES,
+    OUTPUT_FEATURES,
+    PREPROCESSING,
+    TRAINER,
+    TRAINING,
+)
 from ludwig.contribs.mlflow import MlflowCallback
 from ludwig.experiment import experiment_cli
 from ludwig.features.number_feature import numeric_transformation_registry
-from ludwig.globals import TRAINING_PREPROC_FILE_NAME
+from ludwig.globals import DESCRIPTION_FILE_NAME, TRAINING_PREPROC_FILE_NAME
 from ludwig.schema.optimizers import optimizer_registry
 from ludwig.utils.data_utils import load_json, replace_file_extension
 from ludwig.utils.misc_utils import get_from_registry
@@ -41,8 +50,10 @@ def get_feature_configs():
             "name": "y",
             "type": "number",
             "loss": {"type": "mean_squared_error"},
-            "num_fc_layers": 5,
-            "output_size": 64,
+            "decoder": {
+                "num_fc_layers": 5,
+                "fc_output_size": 64,
+            },
         }
     ]
 
@@ -122,7 +133,7 @@ def test_early_stopping(early_stop, generated_data, tmp_path):
 
     # test existence of required files
     train_stats_fp = os.path.join(output_dir, "training_statistics.json")
-    metadata_fp = os.path.join(output_dir, "description.json")
+    metadata_fp = os.path.join(output_dir, DESCRIPTION_FILE_NAME)
     assert os.path.isfile(train_stats_fp)
     assert os.path.isfile(metadata_fp)
 
@@ -370,7 +381,7 @@ def test_regularization(generated_data, tmp_path):
 
         # test existence of required files
         train_stats_fp = os.path.join(output_dir, "training_statistics.json")
-        metadata_fp = os.path.join(output_dir, "description.json")
+        metadata_fp = os.path.join(output_dir, DESCRIPTION_FILE_NAME)
         assert os.path.isfile(train_stats_fp)
         assert os.path.isfile(metadata_fp)
 
@@ -392,17 +403,17 @@ def test_regularization(generated_data, tmp_path):
 # test cache checksum function
 def test_cache_checksum(csv_filename, tmp_path):
     # setup for training
-    input_features = [category_feature(vocab_size=5)]
-    output_features = [category_feature(vocab_size=2, top_k=2)]
+    input_features = [category_feature(encoder={"vocab_size": 5})]
+    output_features = [category_feature(decoder={"vocab_size": 2}, top_k=2)]
 
     source_dataset = os.path.join(tmp_path, csv_filename)
     source_dataset = generate_data(input_features, output_features, source_dataset)
 
     config = {
-        "input_features": input_features,
-        "output_features": output_features,
-        "preprocessing": {"text": {"most_common_word": 1000}},
-        TRAINER: {"epochs": 2},
+        INPUT_FEATURES: input_features,
+        OUTPUT_FEATURES: output_features,
+        DEFAULTS: {CATEGORY: {PREPROCESSING: {"fill_value": "<UNKNOWN>"}}},
+        TRAINER: {EPOCHS: 2},
     }
 
     backend = LocalTestBackend()
@@ -422,9 +433,9 @@ def test_cache_checksum(csv_filename, tmp_path):
     # time stamps should be the same
     assert first_training_timestamp == current_training_timestamp
 
-    # force recreating cache file by changing checksum
+    # force recreating cache file by changing checksum by updating defaults
     prior_training_timestamp = current_training_timestamp
-    config["preprocessing"]["text"]["most_common_word"] = 2000
+    config[DEFAULTS][CATEGORY][PREPROCESSING]["fill_value"] = "<EMPTY>"
     model = LudwigModel(config, backend=backend)
     model.train(dataset=source_dataset, output_directory=output_directory)
     current_training_timestamp = os.path.getmtime(cache_fname)
@@ -444,9 +455,9 @@ def test_cache_checksum(csv_filename, tmp_path):
 
     # force change in feature preprocessing
     prior_training_timestamp = current_training_timestamp
-    input_features = config["input_features"].copy()
-    input_features[0]["preprocessing"] = {"lowercase": True}
-    config["input_features"] = input_features
+    input_features = config[INPUT_FEATURES].copy()
+    input_features[0][PREPROCESSING] = {"lowercase": True}
+    config[INPUT_FEATURES] = input_features
     model = LudwigModel(config, backend=backend)
     model.train(dataset=source_dataset, output_directory=output_directory)
     current_training_timestamp = os.path.getmtime(cache_fname)
@@ -456,9 +467,9 @@ def test_cache_checksum(csv_filename, tmp_path):
 
     # force change in features names (and properties)
     prior_training_timestamp = current_training_timestamp
-    input_features = [category_feature(vocab_size=5), category_feature()]
+    input_features = [category_feature(encoder={"vocab_size": 5}), category_feature()]
     source_dataset = generate_data(input_features, output_features, source_dataset)
-    config["input_features"] = input_features
+    config[INPUT_FEATURES] = input_features
     model = LudwigModel(config, backend=backend)
     model.train(dataset=source_dataset, output_directory=output_directory)
     current_training_timestamp = os.path.getmtime(cache_fname)
