@@ -23,22 +23,45 @@ import pytest
 import torch
 
 from ludwig.api import LudwigModel
-from ludwig.constants import NAME, TRAINER
+from ludwig.constants import ENCODER, NAME, TRAINER
 from tests.integration_tests.utils import (
     binary_feature,
     category_feature,
     generate_data,
-    init_backend,
     RAY_BACKEND_CONFIG,
+    set_feature,
+    text_feature,
 )
 
 
-@pytest.mark.distributed
-@pytest.mark.parametrize("backend", ["local", "ray"])
+def random_binary_logits(*args, num_predict_samples, **kwargs):
+    # Produce an even mix of True and False predictions, as the model may be biased
+    # towards one direction without training
+    return torch.tensor(np.random.uniform(low=-1.0, high=1.0, size=(num_predict_samples,)), dtype=torch.float32)
+
+
+def random_set_logits(*args, num_predict_samples, vocab_size, pct_positive, **kwargs):
+    # Produce a desired mix of predictions based on the pct_positive, as the model may be biased
+    # towards one direction without training
+    num_positive = int(num_predict_samples * pct_positive)
+    num_negative = num_predict_samples - num_positive
+    negative_logits = np.random.uniform(low=-1.0, high=-0.1, size=(num_negative, vocab_size))
+    positive_logits = np.random.uniform(low=0.1, high=1.0, size=(num_positive, vocab_size))
+    logits = np.concatenate([negative_logits, positive_logits], axis=0)
+    return torch.tensor(logits, dtype=torch.float32)  # simulate torch model output
+
+
+@pytest.mark.parametrize(
+    "backend",
+    [
+        pytest.param("local", id="local"),
+        pytest.param("ray", id="ray", marks=pytest.mark.distributed),
+    ],
+)
 @pytest.mark.parametrize("distinct_values", [(False, True), ("No", "Yes")])
-def test_binary_predictions(tmpdir, backend, distinct_values):
+def test_binary_predictions(tmpdir, backend, distinct_values, ray_cluster_2cpu):
     input_features = [
-        category_feature(vocab_size=3),
+        category_feature(encoder={"vocab_size": 3}),
     ]
 
     feature = binary_feature()
@@ -61,7 +84,12 @@ def test_binary_predictions(tmpdir, backend, distinct_values):
 
     config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 1}}
 
-    preds_df = predict_with_backend(tmpdir, config, data_csv_path, backend, num_predict_samples=len(data_df))
+    patch_args = (
+        "ludwig.features.binary_feature.BinaryOutputFeature.logits",
+        partial(random_binary_logits, num_predict_samples=len(data_df)),
+    )
+
+    preds_df, _ = predict_with_backend(tmpdir, config, data_csv_path, backend, patch_args=patch_args)
     cols = set(preds_df.columns)
     assert f"{feature[NAME]}_predictions" in cols
     assert f"{feature[NAME]}_probabilities_{str(false_value)}" in cols
@@ -82,12 +110,17 @@ def test_binary_predictions(tmpdir, backend, distinct_values):
         assert np.allclose(prob_0, 1 - prob_1)
 
 
-@pytest.mark.distributed
-@pytest.mark.parametrize("backend", ["local", "ray"])
+@pytest.mark.parametrize(
+    "backend",
+    [
+        pytest.param("local", id="local"),
+        pytest.param("ray", id="ray", marks=pytest.mark.distributed),
+    ],
+)
 @pytest.mark.parametrize("distinct_values", [(0.0, 1.0), (0, 1)])
-def test_binary_predictions_with_number_dtype(tmpdir, backend, distinct_values):
+def test_binary_predictions_with_number_dtype(tmpdir, backend, distinct_values, ray_cluster_2cpu):
     input_features = [
-        category_feature(vocab_size=3),
+        category_feature(encoder={"vocab_size": 3}),
     ]
 
     feature = binary_feature()
@@ -110,7 +143,12 @@ def test_binary_predictions_with_number_dtype(tmpdir, backend, distinct_values):
 
     config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 1}}
 
-    preds_df = predict_with_backend(tmpdir, config, data_csv_path, backend, num_predict_samples=len(data_df))
+    patch_args = (
+        "ludwig.features.binary_feature.BinaryOutputFeature.logits",
+        partial(random_binary_logits, num_predict_samples=len(data_df)),
+    )
+
+    preds_df, _ = predict_with_backend(tmpdir, config, data_csv_path, backend, patch_args=patch_args)
     cols = set(preds_df.columns)
     assert f"{feature[NAME]}_predictions" in cols
     assert f"{feature[NAME]}_probabilities_False" in cols
@@ -131,32 +169,64 @@ def test_binary_predictions_with_number_dtype(tmpdir, backend, distinct_values):
         assert np.allclose(prob_0, 1 - prob_1)
 
 
-def predict_with_backend(tmpdir, config, data_csv_path, backend, num_predict_samples=None):
-    with init_backend(backend):
-        if backend == "ray":
-            backend = RAY_BACKEND_CONFIG
-            backend["processor"]["type"] = "dask"
+@pytest.mark.parametrize("pct_positive", [1.0, 0.5, 0.0])
+def test_set_feature_saving(tmpdir, pct_positive):
+    backend = "local"
+    input_features = [
+        text_feature(encoder={"vocab_size": 3}),
+    ]
 
-        ludwig_model = LudwigModel(config, backend=backend)
-        _, _, output_directory = ludwig_model.train(
-            dataset=data_csv_path,
-            output_directory=os.path.join(tmpdir, "output"),
-        )
-        # Check that metadata JSON saves and loads correctly
-        ludwig_model = LudwigModel.load(os.path.join(output_directory, "model"))
+    feature = set_feature()
+    output_features = [
+        feature,
+    ]
 
-        # Produce an even mix of True and False predictions, as the model may be biased towards
-        # one direction without training
-        def random_logits(*args, num_predict_samples=None, **kwargs):
-            return torch.tensor(np.random.uniform(low=-1.0, high=1.0, size=(num_predict_samples,)))
+    data_csv_path = generate_data(
+        input_features,
+        output_features,
+        os.path.join(tmpdir, "dataset.csv"),
+        num_examples=100,
+    )
+    data_df = pd.read_csv(data_csv_path)
 
-        if num_predict_samples is not None:
-            with mock.patch(
-                "ludwig.features.binary_feature.BinaryOutputFeature.logits",
-                partial(random_logits, num_predict_samples=num_predict_samples),
-            ):
-                preds_df, _ = ludwig_model.predict(dataset=data_csv_path)
-        else:
+    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 1}}
+
+    patch_args = (
+        "ludwig.features.set_feature.SetOutputFeature.logits",
+        partial(
+            random_set_logits,
+            num_predict_samples=len(data_df),
+            vocab_size=feature[ENCODER]["vocab_size"] + 1,  # +1 for UNK
+            pct_positive=pct_positive,
+        ),
+    )
+
+    preds_df, ludwig_model = predict_with_backend(tmpdir, config, data_csv_path, backend, patch_args=patch_args)
+    cols = set(preds_df.columns)
+    assert f"{feature[NAME]}_predictions" in cols
+    assert f"{feature[NAME]}_probabilities" in cols
+
+    backend = ludwig_model.backend
+    backend.df_engine.to_parquet(preds_df, os.path.join(tmpdir, "preds.parquet"))  # test saving
+
+
+def predict_with_backend(tmpdir, config, data_csv_path, backend, patch_args=None):
+    if backend == "ray":
+        backend = RAY_BACKEND_CONFIG
+        backend["processor"]["type"] = "dask"
+
+    ludwig_model = LudwigModel(config, backend=backend)
+    _, _, output_directory = ludwig_model.train(
+        dataset=data_csv_path,
+        output_directory=os.path.join(tmpdir, "output"),
+    )
+    # Check that metadata JSON saves and loads correctly
+    ludwig_model = LudwigModel.load(os.path.join(output_directory, "model"))
+
+    if patch_args is not None:
+        with mock.patch(*patch_args):
             preds_df, _ = ludwig_model.predict(dataset=data_csv_path)
+    else:
+        preds_df, _ = ludwig_model.predict(dataset=data_csv_path)
 
-    return preds_df
+    return preds_df, ludwig_model
