@@ -87,7 +87,9 @@ except ImportError:
     _modin_ray_incompatible = False
 
 
-def run_api_experiment(config, dataset, backend_config, skip_save_processed_input=True):
+def run_api_experiment(
+    config, dataset, backend_config, predict=False, skip_save_processed_input=True, skip_save_predictions=True
+):
     # Sanity check that we get 4 slots over 1 host
     kwargs = get_trainer_kwargs()
     if torch.cuda.device_count() > 0:
@@ -103,8 +105,9 @@ def run_api_experiment(config, dataset, backend_config, skip_save_processed_inpu
         config,
         dataset=dataset,
         evaluate=True,
-        predict=False,
+        predict=predict,
         skip_save_processed_input=skip_save_processed_input,
+        skip_save_predictions=skip_save_predictions,
     )
 
     assert isinstance(model.backend, RayBackend)
@@ -162,7 +165,9 @@ def run_test_with_features(
     expect_error=False,
     df_engine=None,
     dataset_type="parquet",
+    predict=False,
     skip_save_processed_input=True,
+    skip_save_predictions=True,
     nan_percent=0.0,
     preprocessing=None,
     first_row_none=False,
@@ -195,14 +200,18 @@ def run_test_with_features(
                     config,
                     dataset=dataset,
                     backend_config=backend_config,
+                    predict=predict,
                     skip_save_processed_input=skip_save_processed_input,
+                    skip_save_predictions=skip_save_predictions,
                 )
         else:
             run_fn(
                 config,
                 dataset=dataset,
                 backend_config=backend_config,
+                predict=predict,
                 skip_save_processed_input=skip_save_processed_input,
+                skip_save_predictions=skip_save_predictions,
             )
 
 
@@ -245,11 +254,45 @@ def test_ray_read_binary_files(tmpdir, df_engine, ray_cluster_2cpu):
     assert proc_col.equals(proc_col_expected)
 
 
-@pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
+# TODO(geoffrey): Add dataset_type="csv" back to parameters if we can prevent CI timeouts.
+@pytest.mark.parametrize("dataset_type", ["parquet"])
 @pytest.mark.distributed
-def test_ray_save_processed_input(dataset_type, ray_cluster_2cpu):
+def test_ray_save_inputs_with_nans(tmpdir, dataset_type, ray_cluster_2cpu):
+    image_dest_folder = os.path.join(tmpdir, "generated_images")
+    audio_dest_folder = os.path.join(tmpdir, "generated_audio")
     input_features = [
+        image_feature(
+            folder=image_dest_folder,
+            preprocessing={"in_memory": True, "height": 12, "width": 12, "num_channels": 3, "num_processes": 5},
+            encoder={"output_size": 16, "num_filters": 8},
+        ),
+        audio_feature(
+            folder=audio_dest_folder,
+            preprocessing={
+                "audio_file_length_limit_in_s": 3.0,
+                "missing_value_strategy": BFILL,
+                "in_memory": True,
+                "padding_value": 0,
+                "norm": "per_file",
+                "type": "fbank",
+                "window_length_in_s": 0.04,
+                "window_shift_in_s": 0.02,
+                "num_filter_bands": 80,
+            },
+        ),
+        sequence_feature(encoder={"reduce_output": "sum"}),
         category_feature(encoder={"vocab_size": 2}, reduce_input="sum"),
+        number_feature(normalization="zscore"),
+        set_feature(),
+        binary_feature(),
+        bag_feature(),
+        text_feature(),
+        timeseries_feature(),
+        date_feature(),
+        # TODO: NaN handling not supported. See `test_ray_save_inputs_and_outputs_without_nans` below.
+        # vector_feature(),  # NaNs are not supported by the feature
+        # TODO: feature type not yet supported
+        # h3_feature(),  # ValueError casting large int strings (e.g. '5.864041857092157e+17') to int: MLI-72
     ]
     output_features = [
         category_feature(decoder={"vocab_size": 5}),  # Regression test for #1991 requires multi-class predictions.
@@ -264,6 +307,51 @@ def test_ray_save_processed_input(dataset_type, ray_cluster_2cpu):
     )
 
 
+@pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
+@pytest.mark.distributed
+def test_ray_save_inputs_without_nans(dataset_type, ray_cluster_2cpu):
+    input_features = [
+        vector_feature(),
+    ]
+    output_features = [
+        binary_feature(),
+    ]
+    run_test_with_features(
+        input_features,
+        output_features,
+        df_engine="dask",
+        dataset_type=dataset_type,
+        skip_save_processed_input=False,
+    )
+
+
+@pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
+@pytest.mark.distributed
+def test_ray_save_outputs(dataset_type, ray_cluster_2cpu):
+    input_features = [
+        binary_feature(),
+    ]
+    output_features = [
+        binary_feature(),
+        number_feature(),
+        vector_feature(),
+        # TODO: feature type not yet supported
+        # set_feature(decoder={"vocab_size": 3}),  # Probabilities of set_feature are ragged tensors: MLI-71
+        # sequence_feature(decoder={"vocab_size": 3}),  # Error having to do with a missing key: MLI-70
+        # text_feature(decoder={"vocab_size": 3}),      # Error having to do with a missing key: MLI-70
+    ]
+    # NOTE: This test runs without NaNs because having multiple output features with DROP_ROWS strategy leads to
+    # flakiness in the test having to do with uneven allocation of samples between Ray workers.
+    run_test_with_features(
+        input_features,
+        output_features,
+        df_engine="dask",
+        dataset_type=dataset_type,
+        predict=True,
+        skip_save_predictions=False,
+    )
+
+
 @pytest.mark.distributed
 @pytest.mark.parametrize(
     "df_engine",
