@@ -23,6 +23,7 @@ from ludwig.constants import (
 from ludwig.schema import validate_config
 from ludwig.schema.trainer import ECDTrainerConfig
 from ludwig.utils.backward_compatibility import (
+    _update_backend_cache_credentials,
     _upgrade_encoder_decoder_params,
     _upgrade_feature,
     _upgrade_preprocessing_split,
@@ -509,6 +510,25 @@ def test_update_missing_value_strategy(missing_value_strategy: str):
     assert updated_config == expected_config
 
 
+def test_update_increase_batch_size_on_plateau_max():
+    old_valid_config = {
+        "input_features": [{"name": "input_feature_1", "type": "category"}],
+        "output_features": [{"name": "output_feature_1", "type": "category"}],
+        "trainer": {
+            "increase_batch_size_on_plateau_max": 256,
+        },
+    }
+
+    updated_config = upgrade_to_latest_version(old_valid_config)
+    del updated_config["ludwig_version"]
+
+    expected_config = copy.deepcopy(old_valid_config)
+    del expected_config["trainer"]["increase_batch_size_on_plateau_max"]
+    expected_config["trainer"]["max_batch_size"] = 256
+
+    assert updated_config == expected_config
+
+
 def test_old_class_weights_default():
     old_config = {
         "input_features": [
@@ -638,3 +658,13 @@ def test_upgrade_model_progress_already_valid():
 
     unchanged_model_progress = upgrade_model_progress(valid_model_progress)
     assert unchanged_model_progress == valid_model_progress
+
+
+def test_cache_credentials_backward_compatibility():
+    # From v0.6.3.
+    creds = {"s3": {"client_kwargs": {}}}
+    backend = {"type": "local", "cache_dir": "/foo/bar", "cache_credentials": creds}
+
+    _update_backend_cache_credentials(backend)
+
+    assert backend == {"type": "local", "cache_dir": "/foo/bar", "credentials": {"cache": creds}}
