@@ -14,6 +14,7 @@ from ludwig.constants import (
     FILL_WITH_MODE,
     HYPEROPT,
     INPUT_FEATURES,
+    MAX_POSSIBLE_BATCH_SIZE,
     MISSING_VALUE_STRATEGY,
     MODEL_ECD,
     MODEL_GBM,
@@ -429,6 +430,7 @@ def test_merge_with_defaults():
             "regularization_type": "l2",
             "should_shuffle": True,
             "batch_size": 128,
+            "max_batch_size": MAX_POSSIBLE_BATCH_SIZE,
             "steps_per_checkpoint": 0,
             "checkpoints_per_epoch": 0,
             "reduce_learning_rate_on_plateau": 0.0,
@@ -439,7 +441,6 @@ def test_merge_with_defaults():
             "increase_batch_size_on_plateau": 0,
             "increase_batch_size_on_plateau_patience": 5,
             "increase_batch_size_on_plateau_rate": 2.0,
-            "increase_batch_size_on_plateau_max": 512,
             "increase_batch_size_eval_metric": "loss",
             "increase_batch_size_eval_split": "training",
             "decay": False,
